@@ -2,7 +2,7 @@ import functools
 import sys
 import threading
 import warnings
-from collections import Counter, OrderedDict, defaultdict
+from collections import Counter, defaultdict
 from functools import partial
 
 from django.core.exceptions import AppRegistryNotReady, ImproperlyConfigured
@@ -21,7 +21,7 @@ class Apps:
         # installed_apps is set to None when creating the master registry
         # because it cannot be populated at that point. Other registries must
         # provide a list of installed apps and are populated immediately.
-        if installed_apps is None and hasattr(sys.modules[__name__], 'apps'):
+        if installed_apps is None and hasattr(sys.modules[__name__], "apps"):
             raise RuntimeError("You must supply an installed_apps argument.")
 
         # Mapping of app labels => model names => model classes. Every time a
@@ -31,10 +31,10 @@ class Apps:
         # and whether the registry has been populated. Since it isn't possible
         # to reimport a module safely (it could reexecute initialization code)
         # all_models is never overridden or reset.
-        self.all_models = defaultdict(OrderedDict)
+        self.all_models = defaultdict(dict)
 
         # Mapping of labels to AppConfig instances for installed apps.
-        self.app_configs = OrderedDict()
+        self.app_configs = {}
 
         # Stack of app_configs. Used to store the current state in
         # set_available_apps and set_installed_apps.
@@ -42,6 +42,8 @@ class Apps:
 
         # Whether the registry is populated.
         self.apps_ready = self.models_ready = self.ready = False
+        # For the autoreloader.
+        self.ready_event = threading.Event()
 
         # Lock for thread-safe population.
         self._lock = threading.RLock()
@@ -90,20 +92,22 @@ class Apps:
                 if app_config.label in self.app_configs:
                     raise ImproperlyConfigured(
                         "Application labels aren't unique, "
-                        "duplicates: %s" % app_config.label)
+                        "duplicates: %s" % app_config.label
+                    )
 
                 self.app_configs[app_config.label] = app_config
                 app_config.apps = self
 
             # Check for duplicate app names.
             counts = Counter(
-                app_config.name for app_config in self.app_configs.values())
-            duplicates = [
-                name for name, count in counts.most_common() if count > 1]
+                app_config.name for app_config in self.app_configs.values()
+            )
+            duplicates = [name for name, count in counts.most_common() if count > 1]
             if duplicates:
                 raise ImproperlyConfigured(
                     "Application names aren't unique, "
-                    "duplicates: %s" % ", ".join(duplicates))
+                    "duplicates: %s" % ", ".join(duplicates)
+                )
 
             self.apps_ready = True
 
@@ -120,10 +124,17 @@ class Apps:
                 app_config.ready()
 
             self.ready = True
+            self.ready_event.set()
 
     def check_apps_ready(self):
         """Raise an exception if all apps haven't been imported yet."""
         if not self.apps_ready:
+            from django.conf import settings
+
+            # If "not ready" is due to unconfigured settings, accessing
+            # INSTALLED_APPS raises a more helpful ImproperlyConfigured
+            # exception.
+            settings.INSTALLED_APPS
             raise AppRegistryNotReady("Apps aren't loaded yet.")
 
     def check_models_ready(self):
@@ -171,7 +182,7 @@ class Apps:
 
         result = []
         for app_config in self.app_configs.values():
-            result.extend(list(app_config.get_models(include_auto_created, include_swapped)))
+            result.extend(app_config.get_models(include_auto_created, include_swapped))
         return result
 
     def get_model(self, app_label, model_name=None, require_ready=True):
@@ -192,7 +203,7 @@ class Apps:
             self.check_apps_ready()
 
         if model_name is None:
-            app_label, model_name = app_label.split('.')
+            app_label, model_name = app_label.split(".")
 
         app_config = self.get_app_config(app_label)
 
@@ -208,17 +219,22 @@ class Apps:
         model_name = model._meta.model_name
         app_models = self.all_models[app_label]
         if model_name in app_models:
-            if (model.__name__ == app_models[model_name].__name__ and
-                    model.__module__ == app_models[model_name].__module__):
+            if (
+                model.__name__ == app_models[model_name].__name__
+                and model.__module__ == app_models[model_name].__module__
+            ):
                 warnings.warn(
-                    "Model '%s.%s' was already registered. "
-                    "Reloading models is not advised as it can lead to inconsistencies, "
-                    "most notably with related models." % (app_label, model_name),
-                    RuntimeWarning, stacklevel=2)
+                    "Model '%s.%s' was already registered. Reloading models is not "
+                    "advised as it can lead to inconsistencies, most notably with "
+                    "related models." % (app_label, model_name),
+                    RuntimeWarning,
+                    stacklevel=2,
+                )
             else:
                 raise RuntimeError(
-                    "Conflicting '%s' models in application '%s': %s and %s." %
-                    (model_name, app_label, app_models[model_name], model))
+                    "Conflicting '%s' models in application '%s': %s and %s."
+                    % (model_name, app_label, app_models[model_name], model)
+                )
         app_models[model_name] = model
         self.do_pending_operations(model)
         self.clear_cache()
@@ -245,8 +261,8 @@ class Apps:
         candidates = []
         for app_config in self.app_configs.values():
             if object_name.startswith(app_config.name):
-                subpath = object_name[len(app_config.name):]
-                if subpath == '' or subpath[0] == '.':
+                subpath = object_name[len(app_config.name) :]
+                if subpath == "" or subpath[0] == ".":
                     candidates.append(app_config)
         if candidates:
             return sorted(candidates, key=lambda ac: -len(ac.name))[0]
@@ -261,8 +277,7 @@ class Apps:
         """
         model = self.all_models[app_label].get(model_name.lower())
         if model is None:
-            raise LookupError(
-                "Model '%s.%s' not registered." % (app_label, model_name))
+            raise LookupError("Model '%s.%s' not registered." % (app_label, model_name))
         return model
 
     @functools.lru_cache(maxsize=None)
@@ -277,13 +292,14 @@ class Apps:
         change after Django has loaded the settings, there is no reason to get
         the respective settings attribute over and over again.
         """
+        to_string = to_string.lower()
         for model in self.get_models(include_swapped=True):
             swapped = model._meta.swapped
             # Is this model swapped out for the model given by to_string?
-            if swapped and swapped == to_string:
+            if swapped and swapped.lower() == to_string:
                 return model._meta.swappable
             # Is this model swappable and the one given by to_string?
-            if model._meta.swappable and model._meta.label == to_string:
+            if model._meta.swappable and model._meta.label_lower == to_string:
                 return model._meta.swappable
         return None
 
@@ -308,10 +324,11 @@ class Apps:
             )
 
         self.stored_app_configs.append(self.app_configs)
-        self.app_configs = OrderedDict(
-            (label, app_config)
+        self.app_configs = {
+            label: app_config
             for label, app_config in self.app_configs.items()
-            if app_config.name in available)
+            if app_config.name in available
+        }
         self.clear_cache()
 
     def unset_available_apps(self):
@@ -339,7 +356,7 @@ class Apps:
         if not self.ready:
             raise AppRegistryNotReady("App registry isn't ready yet.")
         self.stored_app_configs.append(self.app_configs)
-        self.app_configs = OrderedDict()
+        self.app_configs = {}
         self.apps_ready = self.models_ready = self.loading = self.ready = False
         self.clear_cache()
         self.populate(installed)
@@ -384,7 +401,7 @@ class Apps:
         # to lazy_model_operation() along with the remaining model args and
         # repeat until all models are loaded and all arguments are applied.
         else:
-            next_model, more_models = model_keys[0], model_keys[1:]
+            next_model, *more_models = model_keys
 
             # This will be executed after the class corresponding to next_model
             # has been imported and registered. The `func` attribute provides
@@ -392,6 +409,7 @@ class Apps:
             def apply_next_model(model):
                 next_function = partial(apply_next_model.func, model)
                 self.lazy_model_operation(next_function, *more_models)
+
             apply_next_model.func = function
 
             # If the model has already been imported and registered, partially
