@@ -10,3 +10,5 @@ class DatabaseFeatures(BaseSpatialFeatures, OracleDatabaseFeatures):
     supports_geometry_field_unique_index = False
     supports_perimeter_geodetic = True
     supports_dwithin_distance_expr = False
+    supports_tolerance_parameter = True
+    unsupported_geojson_options = {"bbox", "crs", "precision"}
