@@ -7,26 +7,45 @@ from ctypes.util import find_library
 from django.contrib.gis.gdal.error import GDALException
 from django.core.exceptions import ImproperlyConfigured
 
-logger = logging.getLogger('django.contrib.gis')
+logger = logging.getLogger("django.contrib.gis")
 
 # Custom library path set?
 try:
     from django.conf import settings
+
     lib_path = settings.GDAL_LIBRARY_PATH
 except (AttributeError, ImportError, ImproperlyConfigured, OSError):
     lib_path = None
 
 if lib_path:
     lib_names = None
-elif os.name == 'nt':
+elif os.name == "nt":
     # Windows NT shared libraries
-    lib_names = ['gdal301', 'gdal300', 'gdal204', 'gdal203', 'gdal202', 'gdal201', 'gdal20']
-elif os.name == 'posix':
+    lib_names = [
+        "gdal303",
+        "gdal302",
+        "gdal301",
+        "gdal300",
+        "gdal204",
+        "gdal203",
+        "gdal202",
+        "gdal201",
+        "gdal20",
+    ]
+elif os.name == "posix":
     # *NIX library names.
     lib_names = [
-        'gdal', 'GDAL',
-        'gdal3.1.0', 'gdal3.0.0',
-        'gdal2.4.0', 'gdal2.3.0', 'gdal2.2.0', 'gdal2.1.0', 'gdal2.0.0',
+        "gdal",
+        "GDAL",
+        "gdal3.3.0",
+        "gdal3.2.0",
+        "gdal3.1.0",
+        "gdal3.0.0",
+        "gdal2.4.0",
+        "gdal2.3.0",
+        "gdal2.2.0",
+        "gdal2.1.0",
+        "gdal2.0.0",
     ]
 else:
     raise ImproperlyConfigured('GDAL is unsupported on OS "%s".' % os.name)
@@ -42,7 +61,7 @@ if lib_names:
 if lib_path is None:
     raise ImproperlyConfigured(
         'Could not find the GDAL library (tried "%s"). Is GDAL installed? '
-        'If it is, try setting GDAL_LIBRARY_PATH in your settings.'
+        "If it is, try setting GDAL_LIBRARY_PATH in your settings."
         % '", "'.join(lib_names)
     )
 
@@ -53,8 +72,9 @@ lgdal = CDLL(lib_path)
 # STDCALL, while others are not.  Thus, the library will also need to
 # be loaded up as WinDLL for said OSR functions that require the
 # different calling convention.
-if os.name == 'nt':
+if os.name == "nt":
     from ctypes import WinDLL
+
     lwingdal = WinDLL(lib_path)
 
 
@@ -63,7 +83,7 @@ def std_call(func):
     Return the correct STDCALL function for certain OSR routines on Win32
     platforms.
     """
-    if os.name == 'nt':
+    if os.name == "nt":
         return lwingdal[func]
     else:
         return lgdal[func]
@@ -72,24 +92,24 @@ def std_call(func):
 # #### Version-information functions. ####
 
 # Return GDAL library version information with the given key.
-_version_info = std_call('GDALVersionInfo')
+_version_info = std_call("GDALVersionInfo")
 _version_info.argtypes = [c_char_p]
 _version_info.restype = c_char_p
 
 
 def gdal_version():
     "Return only the GDAL version number information."
-    return _version_info(b'RELEASE_NAME')
+    return _version_info(b"RELEASE_NAME")
 
 
 def gdal_full_version():
     "Return the full GDAL version information."
-    return _version_info(b'')
+    return _version_info(b"")
 
 
 def gdal_version_info():
     ver = gdal_version()
-    m = re.match(br'^(?P<major>\d+)\.(?P<minor>\d+)(?:\.(?P<subminor>\d+))?', ver)
+    m = re.match(rb"^(?P<major>\d+)\.(?P<minor>\d+)(?:\.(?P<subminor>\d+))?", ver)
     if not m:
         raise GDALException('Could not parse GDAL version string "%s"' % ver)
     major, minor, subminor = m.groups()
@@ -103,7 +123,7 @@ CPLErrorHandler = CFUNCTYPE(None, c_int, c_int, c_char_p)
 
 
 def err_handler(error_class, error_number, message):
-    logger.error('GDAL_ERROR %d: %s', error_number, message)
+    logger.error("GDAL_ERROR %d: %s", error_number, message)
 
 
 err_handler = CPLErrorHandler(err_handler)
@@ -116,5 +136,5 @@ def function(name, args, restype):
     return func
 
 
-set_error_handler = function('CPLSetErrorHandler', [CPLErrorHandler], CPLErrorHandler)
+set_error_handler = function("CPLSetErrorHandler", [CPLErrorHandler], CPLErrorHandler)
 set_error_handler(err_handler)
