@@ -1,19 +1,19 @@
 import socket
-from pathlib import Path
 
 import geoip2.database
 
 from django.conf import settings
 from django.core.exceptions import ValidationError
 from django.core.validators import validate_ipv46_address
+from django.utils._os import to_path
 
 from .resources import City, Country
 
 # Creating the settings dictionary with any settings, if needed.
 GEOIP_SETTINGS = {
-    'GEOIP_PATH': getattr(settings, 'GEOIP_PATH', None),
-    'GEOIP_CITY': getattr(settings, 'GEOIP_CITY', 'GeoLite2-City.mmdb'),
-    'GEOIP_COUNTRY': getattr(settings, 'GEOIP_COUNTRY', 'GeoLite2-Country.mmdb'),
+    "GEOIP_PATH": getattr(settings, "GEOIP_PATH", None),
+    "GEOIP_CITY": getattr(settings, "GEOIP_CITY", "GeoLite2-City.mmdb"),
+    "GEOIP_COUNTRY": getattr(settings, "GEOIP_COUNTRY", "GeoLite2-Country.mmdb"),
 }
 
 
@@ -33,11 +33,13 @@ class GeoIP2:
     MODE_FILE = 4
     # Load database into memory. Pure Python.
     MODE_MEMORY = 8
-    cache_options = frozenset((MODE_AUTO, MODE_MMAP_EXT, MODE_MMAP, MODE_FILE, MODE_MEMORY))
+    cache_options = frozenset(
+        (MODE_AUTO, MODE_MMAP_EXT, MODE_MMAP, MODE_FILE, MODE_MEMORY)
+    )
 
     # Paths to the city & country binary databases.
-    _city_file = ''
-    _country_file = ''
+    _city_file = ""
+    _country_file = ""
 
     # Initially, pointers to GeoIP file references are NULL.
     _city = None
@@ -70,31 +72,31 @@ class GeoIP2:
         if cache in self.cache_options:
             self._cache = cache
         else:
-            raise GeoIP2Exception('Invalid GeoIP caching option: %s' % cache)
+            raise GeoIP2Exception("Invalid GeoIP caching option: %s" % cache)
 
         # Getting the GeoIP data path.
-        path = path or GEOIP_SETTINGS['GEOIP_PATH']
+        path = path or GEOIP_SETTINGS["GEOIP_PATH"]
         if not path:
-            raise GeoIP2Exception('GeoIP path must be provided via parameter or the GEOIP_PATH setting.')
-        if not isinstance(path, str):
-            raise TypeError('Invalid path type: %s' % type(path).__name__)
+            raise GeoIP2Exception(
+                "GeoIP path must be provided via parameter or the GEOIP_PATH setting."
+            )
 
-        path = Path(path)
+        path = to_path(path)
         if path.is_dir():
             # Constructing the GeoIP database filenames using the settings
             # dictionary. If the database files for the GeoLite country
             # and/or city datasets exist, then try to open them.
-            country_db = path / (country or GEOIP_SETTINGS['GEOIP_COUNTRY'])
+            country_db = path / (country or GEOIP_SETTINGS["GEOIP_COUNTRY"])
             if country_db.is_file():
                 self._country = geoip2.database.Reader(str(country_db), mode=cache)
                 self._country_file = country_db
 
-            city_db = path / (city or GEOIP_SETTINGS['GEOIP_CITY'])
+            city_db = path / (city or GEOIP_SETTINGS["GEOIP_CITY"])
             if city_db.is_file():
                 self._city = geoip2.database.Reader(str(city_db), mode=cache)
                 self._city_file = city_db
             if not self._reader:
-                raise GeoIP2Exception('Could not load a database from %s.' % path)
+                raise GeoIP2Exception("Could not load a database from %s." % path)
         elif path.is_file():
             # Otherwise, some detective work will be needed to figure out
             # whether the given database path is for the GeoIP country or city
@@ -102,18 +104,20 @@ class GeoIP2:
             reader = geoip2.database.Reader(str(path), mode=cache)
             db_type = reader.metadata().database_type
 
-            if db_type.endswith('City'):
+            if db_type.endswith("City"):
                 # GeoLite City database detected.
                 self._city = reader
                 self._city_file = path
-            elif db_type.endswith('Country'):
+            elif db_type.endswith("Country"):
                 # GeoIP Country database detected.
                 self._country = reader
                 self._country_file = path
             else:
-                raise GeoIP2Exception('Unable to recognize database edition: %s' % db_type)
+                raise GeoIP2Exception(
+                    "Unable to recognize database edition: %s" % db_type
+                )
         else:
-            raise GeoIP2Exception('GeoIP path must be a valid file or directory.')
+            raise GeoIP2Exception("GeoIP path must be a valid file or directory.")
 
     @property
     def _reader(self):
@@ -133,27 +137,37 @@ class GeoIP2:
 
     def __repr__(self):
         meta = self._reader.metadata()
-        version = '[v%s.%s]' % (meta.binary_format_major_version, meta.binary_format_minor_version)
-        return '<%(cls)s %(version)s _country_file="%(country)s", _city_file="%(city)s">' % {
-            'cls': self.__class__.__name__,
-            'version': version,
-            'country': self._country_file,
-            'city': self._city_file,
-        }
+        version = "[v%s.%s]" % (
+            meta.binary_format_major_version,
+            meta.binary_format_minor_version,
+        )
+        return (
+            '<%(cls)s %(version)s _country_file="%(country)s", _city_file="%(city)s">'
+            % {
+                "cls": self.__class__.__name__,
+                "version": version,
+                "country": self._country_file,
+                "city": self._city_file,
+            }
+        )
 
     def _check_query(self, query, country=False, city=False, city_or_country=False):
         "Check the query and database availability."
         # Making sure a string was passed in for the query.
         if not isinstance(query, str):
-            raise TypeError('GeoIP query must be a string, not type %s' % type(query).__name__)
+            raise TypeError(
+                "GeoIP query must be a string, not type %s" % type(query).__name__
+            )
 
         # Extra checks for the existence of country and city databases.
         if city_or_country and not (self._country or self._city):
-            raise GeoIP2Exception('Invalid GeoIP country and city data files.')
+            raise GeoIP2Exception("Invalid GeoIP country and city data files.")
         elif country and not self._country:
-            raise GeoIP2Exception('Invalid GeoIP country data file: %s' % self._country_file)
+            raise GeoIP2Exception(
+                "Invalid GeoIP country data file: %s" % self._country_file
+            )
         elif city and not self._city:
-            raise GeoIP2Exception('Invalid GeoIP city data file: %s' % self._city_file)
+            raise GeoIP2Exception("Invalid GeoIP city data file: %s" % self._city_file)
 
         # Return the query string back to the caller. GeoIP2 only takes IP addresses.
         try:
@@ -175,12 +189,12 @@ class GeoIP2:
     def country_code(self, query):
         "Return the country code for the given IP Address or FQDN."
         enc_query = self._check_query(query, city_or_country=True)
-        return self.country(enc_query)['country_code']
+        return self.country(enc_query)["country_code"]
 
     def country_name(self, query):
         "Return the country name for the given IP Address or FQDN."
         enc_query = self._check_query(query, city_or_country=True)
-        return self.country(enc_query)['country_name']
+        return self.country(enc_query)["country_name"]
 
     def country(self, query):
         """
@@ -193,7 +207,7 @@ class GeoIP2:
         return Country(self._country_or_city(enc_query))
 
     # #### Coordinate retrieval routines ####
-    def coords(self, query, ordering=('longitude', 'latitude')):
+    def coords(self, query, ordering=("longitude", "latitude")):
         cdict = self.city(query)
         if cdict is None:
             return None
@@ -206,13 +220,14 @@ class GeoIP2:
 
     def lat_lon(self, query):
         "Return a tuple of the (latitude, longitude) for the given query."
-        return self.coords(query, ('latitude', 'longitude'))
+        return self.coords(query, ("latitude", "longitude"))
 
     def geos(self, query):
         "Return a GEOS Point object for the given query."
         ll = self.lon_lat(query)
         if ll:
             from django.contrib.gis.geos import Point
+
             return Point(ll, srid=4326)
         else:
             return None
@@ -222,7 +237,10 @@ class GeoIP2:
     def info(self):
         "Return information about the GeoIP library and databases in use."
         meta = self._reader.metadata()
-        return 'GeoIP Library:\n\t%s.%s\n' % (meta.binary_format_major_version, meta.binary_format_minor_version)
+        return "GeoIP Library:\n\t%s.%s\n" % (
+            meta.binary_format_major_version,
+            meta.binary_format_minor_version,
+        )
 
     @classmethod
     def open(cls, full_path, cache):
