@@ -1,16 +1,17 @@
-from psycopg2.extras import (
-    DateRange, DateTimeRange, DateTimeTZRange, NumericRange,
-)
+from psycopg2.extras import DateRange, DateTimeRange, DateTimeTZRange, NumericRange
 
 from django.apps import AppConfig
 from django.db import connections
 from django.db.backends.signals import connection_created
 from django.db.migrations.writer import MigrationWriter
-from django.db.models import CharField, TextField
+from django.db.models import CharField, OrderBy, TextField
+from django.db.models.functions import Collate
+from django.db.models.indexes import IndexExpression
 from django.test.signals import setting_changed
 from django.utils.translation import gettext_lazy as _
 
-from .lookups import SearchLookup, TrigramSimilar, Unaccent
+from .indexes import OpClass
+from .lookups import SearchLookup, TrigramSimilar, TrigramWordSimilar, Unaccent
 from .serializers import RangeSerializer
 from .signals import register_type_handlers
 
@@ -22,7 +23,11 @@ def uninstall_if_needed(setting, value, enter, **kwargs):
     Undo the effects of PostgresConfig.ready() when django.contrib.postgres
     is "uninstalled" by override_settings().
     """
-    if not enter and setting == 'INSTALLED_APPS' and 'django.contrib.postgres' not in set(value):
+    if (
+        not enter
+        and setting == "INSTALLED_APPS"
+        and "django.contrib.postgres" not in set(value)
+    ):
         connection_created.disconnect(register_type_handlers)
         CharField._unregister_lookup(Unaccent)
         TextField._unregister_lookup(Unaccent)
@@ -30,6 +35,8 @@ def uninstall_if_needed(setting, value, enter, **kwargs):
         TextField._unregister_lookup(SearchLookup)
         CharField._unregister_lookup(TrigramSimilar)
         TextField._unregister_lookup(TrigramSimilar)
+        CharField._unregister_lookup(TrigramWordSimilar)
+        TextField._unregister_lookup(TrigramWordSimilar)
         # Disconnect this receiver until the next time this app is installed
         # and ready() connects it again to prevent unnecessary processing on
         # each setting change.
@@ -38,22 +45,23 @@ def uninstall_if_needed(setting, value, enter, **kwargs):
 
 
 class PostgresConfig(AppConfig):
-    name = 'django.contrib.postgres'
-    verbose_name = _('PostgreSQL extensions')
+    name = "django.contrib.postgres"
+    verbose_name = _("PostgreSQL extensions")
 
     def ready(self):
         setting_changed.connect(uninstall_if_needed)
         # Connections may already exist before we are called.
         for conn in connections.all():
-            if conn.vendor == 'postgresql':
-                conn.introspection.data_types_reverse.update({
-                    3802: 'django.contrib.postgres.fields.JSONField',
-                    3904: 'django.contrib.postgres.fields.IntegerRangeField',
-                    3906: 'django.contrib.postgres.fields.DecimalRangeField',
-                    3910: 'django.contrib.postgres.fields.DateTimeRangeField',
-                    3912: 'django.contrib.postgres.fields.DateRangeField',
-                    3926: 'django.contrib.postgres.fields.BigIntegerRangeField',
-                })
+            if conn.vendor == "postgresql":
+                conn.introspection.data_types_reverse.update(
+                    {
+                        3904: "django.contrib.postgres.fields.IntegerRangeField",
+                        3906: "django.contrib.postgres.fields.DecimalRangeField",
+                        3910: "django.contrib.postgres.fields.DateTimeRangeField",
+                        3912: "django.contrib.postgres.fields.DateRangeField",
+                        3926: "django.contrib.postgres.fields.BigIntegerRangeField",
+                    }
+                )
                 if conn.connection is not None:
                     register_type_handlers(conn)
         connection_created.connect(register_type_handlers)
@@ -63,4 +71,7 @@ class PostgresConfig(AppConfig):
         TextField.register_lookup(SearchLookup)
         CharField.register_lookup(TrigramSimilar)
         TextField.register_lookup(TrigramSimilar)
+        CharField.register_lookup(TrigramWordSimilar)
+        TextField.register_lookup(TrigramWordSimilar)
         MigrationWriter.register_serializer(RANGE_TYPES, RangeSerializer)
+        IndexExpression.register_wrappers(OrderBy, OpClass, Collate)
