@@ -10,17 +10,23 @@ from django.db.models.lookups import PostgresOperatorLookup
 from .utils import AttributeSetter
 
 __all__ = [
-    'RangeField', 'IntegerRangeField', 'BigIntegerRangeField',
-    'DecimalRangeField', 'DateTimeRangeField', 'DateRangeField',
-    'RangeBoundary', 'RangeOperators',
+    "RangeField",
+    "IntegerRangeField",
+    "BigIntegerRangeField",
+    "DecimalRangeField",
+    "DateTimeRangeField",
+    "DateRangeField",
+    "RangeBoundary",
+    "RangeOperators",
 ]
 
 
 class RangeBoundary(models.Expression):
     """A class that represents range boundaries."""
+
     def __init__(self, inclusive_lower=True, inclusive_upper=False):
-        self.lower = '[' if inclusive_lower else '('
-        self.upper = ']' if inclusive_upper else ')'
+        self.lower = "[" if inclusive_lower else "("
+        self.upper = "]" if inclusive_upper else ")"
 
     def as_sql(self, compiler, connection):
         return "'%s%s'" % (self.lower, self.upper), []
@@ -28,37 +34,40 @@ class RangeBoundary(models.Expression):
 
 class RangeOperators:
     # https://www.postgresql.org/docs/current/functions-range.html#RANGE-OPERATORS-TABLE
-    EQUAL = '='
-    NOT_EQUAL = '<>'
-    CONTAINS = '@>'
-    CONTAINED_BY = '<@'
-    OVERLAPS = '&&'
-    FULLY_LT = '<<'
-    FULLY_GT = '>>'
-    NOT_LT = '&>'
-    NOT_GT = '&<'
-    ADJACENT_TO = '-|-'
+    EQUAL = "="
+    NOT_EQUAL = "<>"
+    CONTAINS = "@>"
+    CONTAINED_BY = "<@"
+    OVERLAPS = "&&"
+    FULLY_LT = "<<"
+    FULLY_GT = ">>"
+    NOT_LT = "&>"
+    NOT_GT = "&<"
+    ADJACENT_TO = "-|-"
 
 
 class RangeField(models.Field):
     empty_strings_allowed = False
 
     def __init__(self, *args, **kwargs):
-        # Initializing base_field here ensures that its model matches the model for self.
-        if hasattr(self, 'base_field'):
+        # Initializing base_field here ensures that its model matches the model
+        # for self.
+        if hasattr(self, "base_field"):
             self.base_field = self.base_field()
         super().__init__(*args, **kwargs)
 
     @property
     def model(self):
         try:
-            return self.__dict__['model']
+            return self.__dict__["model"]
         except KeyError:
-            raise AttributeError("'%s' object has no attribute 'model'" % self.__class__.__name__)
+            raise AttributeError(
+                "'%s' object has no attribute 'model'" % self.__class__.__name__
+            )
 
     @model.setter
     def model(self, model):
-        self.__dict__['model'] = model
+        self.__dict__["model"] = model
         self.base_field.model = model
 
     @classmethod
@@ -78,7 +87,7 @@ class RangeField(models.Field):
         if isinstance(value, str):
             # Assume we're deserializing
             vals = json.loads(value)
-            for end in ('lower', 'upper'):
+            for end in ("lower", "upper"):
                 if end in vals:
                     vals[end] = self.base_field.to_python(vals[end])
             value = self.range_type(**vals)
@@ -98,7 +107,7 @@ class RangeField(models.Field):
             return json.dumps({"empty": True})
         base_field = self.base_field
         result = {"bounds": value._bounds}
-        for end in ('lower', 'upper'):
+        for end in ("lower", "upper"):
             val = getattr(value, end)
             if val is None:
                 result[end] = None
@@ -108,7 +117,7 @@ class RangeField(models.Field):
         return json.dumps(result)
 
     def formfield(self, **kwargs):
-        kwargs.setdefault('form_class', self.form_field)
+        kwargs.setdefault("form_class", self.form_field)
         return super().formfield(**kwargs)
 
 
@@ -118,7 +127,7 @@ class IntegerRangeField(RangeField):
     form_field = forms.IntegerRangeField
 
     def db_type(self, connection):
-        return 'int4range'
+        return "int4range"
 
 
 class BigIntegerRangeField(RangeField):
@@ -127,7 +136,7 @@ class BigIntegerRangeField(RangeField):
     form_field = forms.IntegerRangeField
 
     def db_type(self, connection):
-        return 'int8range'
+        return "int8range"
 
 
 class DecimalRangeField(RangeField):
@@ -136,7 +145,7 @@ class DecimalRangeField(RangeField):
     form_field = forms.DecimalRangeField
 
     def db_type(self, connection):
-        return 'numrange'
+        return "numrange"
 
 
 class DateTimeRangeField(RangeField):
@@ -145,7 +154,7 @@ class DateTimeRangeField(RangeField):
     form_field = forms.DateTimeRangeField
 
     def db_type(self, connection):
-        return 'tstzrange'
+        return "tstzrange"
 
 
 class DateRangeField(RangeField):
@@ -154,7 +163,7 @@ class DateRangeField(RangeField):
     form_field = forms.DateRangeField
 
     def db_type(self, connection):
-        return 'daterange'
+        return "daterange"
 
 
 RangeField.register_lookup(lookups.DataContains)
@@ -167,30 +176,33 @@ class DateTimeRangeContains(PostgresOperatorLookup):
     Lookup for Date/DateTimeRange containment to cast the rhs to the correct
     type.
     """
-    lookup_name = 'contains'
+
+    lookup_name = "contains"
     postgres_operator = RangeOperators.CONTAINS
 
     def process_rhs(self, compiler, connection):
         # Transform rhs value for db lookup.
         if isinstance(self.rhs, datetime.date):
-            output_field = models.DateTimeField() if isinstance(self.rhs, datetime.datetime) else models.DateField()
-            value = models.Value(self.rhs, output_field=output_field)
+            value = models.Value(self.rhs)
             self.rhs = value.resolve_expression(compiler.query)
         return super().process_rhs(compiler, connection)
 
     def as_postgresql(self, compiler, connection):
         sql, params = super().as_postgresql(compiler, connection)
         # Cast the rhs if needed.
-        cast_sql = ''
+        cast_sql = ""
         if (
-            isinstance(self.rhs, models.Expression) and
-            self.rhs._output_field_or_none and
+            isinstance(self.rhs, models.Expression)
+            and self.rhs._output_field_or_none
+            and
             # Skip cast if rhs has a matching range type.
-            not isinstance(self.rhs._output_field_or_none, self.lhs.output_field.__class__)
+            not isinstance(
+                self.rhs._output_field_or_none, self.lhs.output_field.__class__
+            )
         ):
             cast_internal_type = self.lhs.output_field.base_field.get_internal_type()
-            cast_sql = '::{}'.format(connection.data_types.get(cast_internal_type))
-        return '%s%s' % (sql, cast_sql), params
+            cast_sql = "::{}".format(connection.data_types.get(cast_internal_type))
+        return "%s%s" % (sql, cast_sql), params
 
 
 DateRangeField.register_lookup(DateTimeRangeContains)
@@ -198,31 +210,31 @@ DateTimeRangeField.register_lookup(DateTimeRangeContains)
 
 
 class RangeContainedBy(PostgresOperatorLookup):
-    lookup_name = 'contained_by'
+    lookup_name = "contained_by"
     type_mapping = {
-        'smallint': 'int4range',
-        'integer': 'int4range',
-        'bigint': 'int8range',
-        'double precision': 'numrange',
-        'numeric': 'numrange',
-        'date': 'daterange',
-        'timestamp with time zone': 'tstzrange',
+        "smallint": "int4range",
+        "integer": "int4range",
+        "bigint": "int8range",
+        "double precision": "numrange",
+        "numeric": "numrange",
+        "date": "daterange",
+        "timestamp with time zone": "tstzrange",
     }
     postgres_operator = RangeOperators.CONTAINED_BY
 
     def process_rhs(self, compiler, connection):
         rhs, rhs_params = super().process_rhs(compiler, connection)
         # Ignore precision for DecimalFields.
-        db_type = self.lhs.output_field.cast_db_type(connection).split('(')[0]
+        db_type = self.lhs.output_field.cast_db_type(connection).split("(")[0]
         cast_type = self.type_mapping[db_type]
-        return '%s::%s' % (rhs, cast_type), rhs_params
+        return "%s::%s" % (rhs, cast_type), rhs_params
 
     def process_lhs(self, compiler, connection):
         lhs, lhs_params = super().process_lhs(compiler, connection)
         if isinstance(self.lhs.output_field, models.FloatField):
-            lhs = '%s::numeric' % lhs
+            lhs = "%s::numeric" % lhs
         elif isinstance(self.lhs.output_field, models.SmallIntegerField):
-            lhs = '%s::integer' % lhs
+            lhs = "%s::integer" % lhs
         return lhs, lhs_params
 
     def get_prep_lookup(self):
@@ -238,38 +250,38 @@ models.DecimalField.register_lookup(RangeContainedBy)
 
 @RangeField.register_lookup
 class FullyLessThan(PostgresOperatorLookup):
-    lookup_name = 'fully_lt'
+    lookup_name = "fully_lt"
     postgres_operator = RangeOperators.FULLY_LT
 
 
 @RangeField.register_lookup
 class FullGreaterThan(PostgresOperatorLookup):
-    lookup_name = 'fully_gt'
+    lookup_name = "fully_gt"
     postgres_operator = RangeOperators.FULLY_GT
 
 
 @RangeField.register_lookup
 class NotLessThan(PostgresOperatorLookup):
-    lookup_name = 'not_lt'
+    lookup_name = "not_lt"
     postgres_operator = RangeOperators.NOT_LT
 
 
 @RangeField.register_lookup
 class NotGreaterThan(PostgresOperatorLookup):
-    lookup_name = 'not_gt'
+    lookup_name = "not_gt"
     postgres_operator = RangeOperators.NOT_GT
 
 
 @RangeField.register_lookup
 class AdjacentToLookup(PostgresOperatorLookup):
-    lookup_name = 'adjacent_to'
+    lookup_name = "adjacent_to"
     postgres_operator = RangeOperators.ADJACENT_TO
 
 
 @RangeField.register_lookup
 class RangeStartsWith(models.Transform):
-    lookup_name = 'startswith'
-    function = 'lower'
+    lookup_name = "startswith"
+    function = "lower"
 
     @property
     def output_field(self):
@@ -278,8 +290,8 @@ class RangeStartsWith(models.Transform):
 
 @RangeField.register_lookup
 class RangeEndsWith(models.Transform):
-    lookup_name = 'endswith'
-    function = 'upper'
+    lookup_name = "endswith"
+    function = "upper"
 
     @property
     def output_field(self):
@@ -288,34 +300,34 @@ class RangeEndsWith(models.Transform):
 
 @RangeField.register_lookup
 class IsEmpty(models.Transform):
-    lookup_name = 'isempty'
-    function = 'isempty'
+    lookup_name = "isempty"
+    function = "isempty"
     output_field = models.BooleanField()
 
 
 @RangeField.register_lookup
 class LowerInclusive(models.Transform):
-    lookup_name = 'lower_inc'
-    function = 'LOWER_INC'
+    lookup_name = "lower_inc"
+    function = "LOWER_INC"
     output_field = models.BooleanField()
 
 
 @RangeField.register_lookup
 class LowerInfinite(models.Transform):
-    lookup_name = 'lower_inf'
-    function = 'LOWER_INF'
+    lookup_name = "lower_inf"
+    function = "LOWER_INF"
     output_field = models.BooleanField()
 
 
 @RangeField.register_lookup
 class UpperInclusive(models.Transform):
-    lookup_name = 'upper_inc'
-    function = 'UPPER_INC'
+    lookup_name = "upper_inc"
+    function = "UPPER_INC"
     output_field = models.BooleanField()
 
 
 @RangeField.register_lookup
 class UpperInfinite(models.Transform):
-    lookup_name = 'upper_inf'
-    function = 'UPPER_INF'
+    lookup_name = "upper_inf"
+    function = "UPPER_INF"
     output_field = models.BooleanField()
