@@ -1,20 +1,12 @@
-import base64
 import logging
 import string
-import warnings
 from datetime import datetime, timedelta
 
 from django.conf import settings
-from django.contrib.sessions.exceptions import SuspiciousSession
 from django.core import signing
-from django.core.exceptions import SuspiciousOperation
 from django.utils import timezone
-from django.utils.crypto import (
-    constant_time_compare, get_random_string, salted_hmac,
-)
-from django.utils.deprecation import RemovedInDjango40Warning
+from django.utils.crypto import get_random_string
 from django.utils.module_loading import import_string
-from django.utils.translation import LANGUAGE_SESSION_KEY
 
 # session_key should not be case sensitive because some backends can store it
 # on case insensitive file systems.
@@ -26,6 +18,7 @@ class CreateError(Exception):
     Used internally as a consistent exception type to catch from save (see the
     docstring for SessionBase.save() for details).
     """
+
     pass
 
 
@@ -33,6 +26,7 @@ class UpdateError(Exception):
     """
     Occurs if Django tries to update a session that was deleted.
     """
+
     pass
 
 
@@ -40,8 +34,9 @@ class SessionBase:
     """
     Base class for all Session classes.
     """
-    TEST_COOKIE_NAME = 'testcookie'
-    TEST_COOKIE_VALUE = 'worked'
+
+    TEST_COOKIE_NAME = "testcookie"
+    TEST_COOKIE_VALUE = "worked"
 
     __not_given = object()
 
@@ -55,13 +50,6 @@ class SessionBase:
         return key in self._session
 
     def __getitem__(self, key):
-        if key == LANGUAGE_SESSION_KEY:
-            warnings.warn(
-                'The user language will no longer be stored in '
-                'request.session in Django 4.0. Read it from '
-                'request.COOKIES[settings.LANGUAGE_COOKIE_NAME] instead.',
-                RemovedInDjango40Warning, stacklevel=2,
-            )
         return self._session[key]
 
     def __setitem__(self, key, value):
@@ -74,7 +62,7 @@ class SessionBase:
 
     @property
     def key_salt(self):
-        return 'django.contrib.sessions.' + self.__class__.__qualname__
+        return "django.contrib.sessions." + self.__class__.__qualname__
 
     def get(self, key, default=None):
         return self._session.get(key, default)
@@ -101,44 +89,28 @@ class SessionBase:
     def delete_test_cookie(self):
         del self[self.TEST_COOKIE_NAME]
 
-    def _hash(self, value):
-        # RemovedInDjango40Warning: pre-Django 3.1 format will be invalid.
-        key_salt = "django.contrib.sessions" + self.__class__.__name__
-        return salted_hmac(key_salt, value).hexdigest()
-
     def encode(self, session_dict):
         "Return the given session dictionary serialized and encoded as a string."
         return signing.dumps(
-            session_dict, salt=self.key_salt, serializer=self.serializer,
+            session_dict,
+            salt=self.key_salt,
+            serializer=self.serializer,
             compress=True,
         )
 
     def decode(self, session_data):
         try:
-            return signing.loads(session_data, salt=self.key_salt, serializer=self.serializer)
-        # RemovedInDjango40Warning: when the deprecation ends, handle here
-        # exceptions similar to what _legacy_decode() does now.
+            return signing.loads(
+                session_data, salt=self.key_salt, serializer=self.serializer
+            )
+        except signing.BadSignature:
+            logger = logging.getLogger("django.security.SuspiciousSession")
+            logger.warning("Session data corrupted")
         except Exception:
-            return self._legacy_decode(session_data)
-
-    def _legacy_decode(self, session_data):
-        # RemovedInDjango40Warning: pre-Django 3.1 format will be invalid.
-        encoded_data = base64.b64decode(session_data.encode('ascii'))
-        try:
-            # could produce ValueError if there is no ':'
-            hash, serialized = encoded_data.split(b':', 1)
-            expected_hash = self._hash(serialized)
-            if not constant_time_compare(hash.decode(), expected_hash):
-                raise SuspiciousSession("Session data corrupted")
-            else:
-                return self.serializer().loads(serialized)
-        except Exception as e:
-            # ValueError, SuspiciousOperation, unpickling exceptions. If any of
-            # these happen, just return an empty dictionary (an empty session).
-            if isinstance(e, SuspiciousOperation):
-                logger = logging.getLogger('django.security.%s' % e.__class__.__name__)
-                logger.warning(str(e))
-            return {}
+            # ValueError, unpickling exceptions. If any of these happen, just
+            # return an empty dictionary (an empty session).
+            pass
+        return {}
 
     def update(self, dict_):
         self._session.update(dict_)
@@ -232,18 +204,18 @@ class SessionBase:
         arguments specifying the modification and expiry of the session.
         """
         try:
-            modification = kwargs['modification']
+            modification = kwargs["modification"]
         except KeyError:
             modification = timezone.now()
         # Make the difference between "expiry=None passed in kwargs" and
         # "expiry not passed in kwargs", in order to guarantee not to trigger
         # self.load() when expiry is provided.
         try:
-            expiry = kwargs['expiry']
+            expiry = kwargs["expiry"]
         except KeyError:
-            expiry = self.get('_session_expiry')
+            expiry = self.get("_session_expiry")
 
-        if not expiry:   # Checks both None and 0 cases
+        if not expiry:  # Checks both None and 0 cases
             return self.get_session_cookie_age()
         if not isinstance(expiry, datetime):
             return expiry
@@ -257,14 +229,14 @@ class SessionBase:
         arguments specifying the modification and expiry of the session.
         """
         try:
-            modification = kwargs['modification']
+            modification = kwargs["modification"]
         except KeyError:
             modification = timezone.now()
         # Same comment as in get_expiry_age
         try:
-            expiry = kwargs['expiry']
+            expiry = kwargs["expiry"]
         except KeyError:
-            expiry = self.get('_session_expiry')
+            expiry = self.get("_session_expiry")
 
         if isinstance(expiry, datetime):
             return expiry
@@ -289,13 +261,13 @@ class SessionBase:
         if value is None:
             # Remove any custom expiration for this session.
             try:
-                del self['_session_expiry']
+                del self["_session_expiry"]
             except KeyError:
                 pass
             return
         if isinstance(value, timedelta):
             value = timezone.now() + value
-        self['_session_expiry'] = value
+        self["_session_expiry"] = value
 
     def get_expire_at_browser_close(self):
         """
@@ -304,9 +276,9 @@ class SessionBase:
         ``get_expiry_date()`` or ``get_expiry_age()`` to find the actual expiry
         date/age, if there is one.
         """
-        if self.get('_session_expiry') is None:
+        if self.get("_session_expiry") is None:
             return settings.SESSION_EXPIRE_AT_BROWSER_CLOSE
-        return self.get('_session_expiry') == 0
+        return self.get("_session_expiry") == 0
 
     def flush(self):
         """
@@ -334,7 +306,9 @@ class SessionBase:
         """
         Return True if the given session_key already exists.
         """
-        raise NotImplementedError('subclasses of SessionBase must provide an exists() method')
+        raise NotImplementedError(
+            "subclasses of SessionBase must provide an exists() method"
+        )
 
     def create(self):
         """
@@ -342,7 +316,9 @@ class SessionBase:
         a unique key and will have saved the result once (with empty data)
         before the method returns.
         """
-        raise NotImplementedError('subclasses of SessionBase must provide a create() method')
+        raise NotImplementedError(
+            "subclasses of SessionBase must provide a create() method"
+        )
 
     def save(self, must_create=False):
         """
@@ -350,20 +326,26 @@ class SessionBase:
         object (or raise CreateError). Otherwise, only update an existing
         object and don't create one (raise UpdateError if needed).
         """
-        raise NotImplementedError('subclasses of SessionBase must provide a save() method')
+        raise NotImplementedError(
+            "subclasses of SessionBase must provide a save() method"
+        )
 
     def delete(self, session_key=None):
         """
         Delete the session data under this key. If the key is None, use the
         current session key value.
         """
-        raise NotImplementedError('subclasses of SessionBase must provide a delete() method')
+        raise NotImplementedError(
+            "subclasses of SessionBase must provide a delete() method"
+        )
 
     def load(self):
         """
         Load the session data and return a dictionary.
         """
-        raise NotImplementedError('subclasses of SessionBase must provide a load() method')
+        raise NotImplementedError(
+            "subclasses of SessionBase must provide a load() method"
+        )
 
     @classmethod
     def clear_expired(cls):
@@ -374,4 +356,4 @@ class SessionBase:
         NotImplementedError. If it isn't necessary, because the backend has
         a built-in expiration mechanism, it should be a no-op.
         """
-        raise NotImplementedError('This backend does not support clear_expired().')
+        raise NotImplementedError("This backend does not support clear_expired().")
