@@ -6,7 +6,10 @@ import tempfile
 
 from django.conf import settings
 from django.contrib.sessions.backends.base import (
-    VALID_KEY_CHARS, CreateError, SessionBase, UpdateError,
+    VALID_KEY_CHARS,
+    CreateError,
+    SessionBase,
+    UpdateError,
 )
 from django.contrib.sessions.exceptions import InvalidSessionKey
 from django.core.exceptions import ImproperlyConfigured, SuspiciousOperation
@@ -17,8 +20,9 @@ class SessionStore(SessionBase):
     """
     Implement a file based session store.
     """
+
     def __init__(self, session_key=None):
-        self.storage_path = type(self)._get_storage_path()
+        self.storage_path = self._get_storage_path()
         self.file_prefix = settings.SESSION_COOKIE_NAME
         super().__init__(session_key)
 
@@ -27,16 +31,16 @@ class SessionStore(SessionBase):
         try:
             return cls._storage_path
         except AttributeError:
-            storage_path = getattr(settings, "SESSION_FILE_PATH", None)
-            if not storage_path:
-                storage_path = tempfile.gettempdir()
-
+            storage_path = (
+                getattr(settings, "SESSION_FILE_PATH", None) or tempfile.gettempdir()
+            )
             # Make sure the storage path is valid.
             if not os.path.isdir(storage_path):
                 raise ImproperlyConfigured(
                     "The session storage path %r doesn't exist. Please set your"
                     " SESSION_FILE_PATH setting to an existing directory in which"
-                    " Django can store session data." % storage_path)
+                    " Django can store session data." % storage_path
+                )
 
             cls._storage_path = storage_path
             return storage_path
@@ -52,8 +56,7 @@ class SessionStore(SessionBase):
         # should always be md5s, so they should never contain directory
         # components.
         if not set(session_key).issubset(VALID_KEY_CHARS):
-            raise InvalidSessionKey(
-                "Invalid characters in session key")
+            raise InvalidSessionKey("Invalid characters in session key")
 
         return os.path.join(self.storage_path, self.file_prefix + session_key)
 
@@ -62,26 +65,22 @@ class SessionStore(SessionBase):
         Return the modification time of the file storing the session's content.
         """
         modification = os.stat(self._key_to_file()).st_mtime
-        if settings.USE_TZ:
-            modification = datetime.datetime.utcfromtimestamp(modification)
-            modification = modification.replace(tzinfo=timezone.utc)
-        else:
-            modification = datetime.datetime.fromtimestamp(modification)
-        return modification
+        tz = timezone.utc if settings.USE_TZ else None
+        return datetime.datetime.fromtimestamp(modification, tz=tz)
 
     def _expiry_date(self, session_data):
         """
         Return the expiry time of the file storing the session's content.
         """
-        expiry = session_data.get('_session_expiry')
-        if not expiry:
-            expiry = self._last_modification() + datetime.timedelta(seconds=settings.SESSION_COOKIE_AGE)
-        return expiry
+        return session_data.get("_session_expiry") or (
+            self._last_modification()
+            + datetime.timedelta(seconds=self.get_session_cookie_age())
+        )
 
     def load(self):
         session_data = {}
         try:
-            with open(self._key_to_file(), "rb") as session_file:
+            with open(self._key_to_file(), encoding="ascii") as session_file:
                 file_data = session_file.read()
             # Don't fail if there is no data in the session file.
             # We may have opened the empty placeholder file.
@@ -90,7 +89,9 @@ class SessionStore(SessionBase):
                     session_data = self.decode(file_data)
                 except (EOFError, SuspiciousOperation) as e:
                     if isinstance(e, SuspiciousOperation):
-                        logger = logging.getLogger('django.security.%s' % e.__class__.__name__)
+                        logger = logging.getLogger(
+                            "django.security.%s" % e.__class__.__name__
+                        )
                         logger.warning(str(e))
                     self.create()
 
@@ -100,7 +101,7 @@ class SessionStore(SessionBase):
                     session_data = {}
                     self.delete()
                     self.create()
-        except (IOError, SuspiciousOperation):
+        except (OSError, SuspiciousOperation):
             self._session_key = None
         return session_data
 
@@ -126,7 +127,7 @@ class SessionStore(SessionBase):
         try:
             # Make sure the file exists.  If it does not already exist, an
             # empty placeholder file is created.
-            flags = os.O_WRONLY | getattr(os, 'O_BINARY', 0)
+            flags = os.O_WRONLY | getattr(os, "O_BINARY", 0)
             if must_create:
                 flags |= os.O_EXCL | os.O_CREAT
             fd = os.open(session_file_name, flags)
@@ -156,7 +157,9 @@ class SessionStore(SessionBase):
         dir, prefix = os.path.split(session_file_name)
 
         try:
-            output_file_fd, output_file_name = tempfile.mkstemp(dir=dir, prefix=prefix + '_out_')
+            output_file_fd, output_file_name = tempfile.mkstemp(
+                dir=dir, prefix=prefix + "_out_"
+            )
             renamed = False
             try:
                 try:
@@ -172,7 +175,7 @@ class SessionStore(SessionBase):
             finally:
                 if not renamed:
                     os.unlink(output_file_name)
-        except (OSError, IOError, EOFError):
+        except (EOFError, OSError):
             pass
 
     def exists(self, session_key):
@@ -199,7 +202,7 @@ class SessionStore(SessionBase):
         for session_file in os.listdir(storage_path):
             if not session_file.startswith(file_prefix):
                 continue
-            session_key = session_file[len(file_prefix):]
+            session_key = session_file[len(file_prefix) :]
             session = cls(session_key)
             # When an expired session is loaded, its file is removed, and a
             # new file is immediately created. Prevent this by disabling
