@@ -1,5 +1,6 @@
 from itertools import chain
 
+from django.utils.inspect import func_accepts_kwargs
 from django.utils.itercompat import is_iterable
 
 
@@ -7,22 +8,24 @@ class Tags:
     """
     Built-in tags for internal checks.
     """
-    admin = 'admin'
-    async_support = 'async_support'
-    caches = 'caches'
-    compatibility = 'compatibility'
-    database = 'database'
-    models = 'models'
-    security = 'security'
-    signals = 'signals'
-    staticfiles = 'staticfiles'
-    templates = 'templates'
-    translation = 'translation'
-    urls = 'urls'
+
+    admin = "admin"
+    async_support = "async_support"
+    caches = "caches"
+    compatibility = "compatibility"
+    database = "database"
+    files = "files"
+    models = "models"
+    security = "security"
+    signals = "signals"
+    sites = "sites"
+    staticfiles = "staticfiles"
+    templates = "templates"
+    translation = "translation"
+    urls = "urls"
 
 
 class CheckRegistry:
-
     def __init__(self):
         self.registered_checks = set()
         self.deployment_checks = set()
@@ -37,15 +40,24 @@ class CheckRegistry:
 
             registry = CheckRegistry()
             @registry.register('mytag', 'anothertag')
-            def my_check(apps, **kwargs):
+            def my_check(app_configs, **kwargs):
                 # ... perform checks and collect `errors` ...
                 return errors
             # or
             registry.register(my_check, 'mytag', 'anothertag')
         """
+
         def inner(check):
+            if not func_accepts_kwargs(check):
+                raise TypeError(
+                    "Check functions must accept keyword arguments (**kwargs)."
+                )
             check.tags = tags
-            checks = self.deployment_checks if kwargs.get('deploy') else self.registered_checks
+            checks = (
+                self.deployment_checks
+                if kwargs.get("deploy")
+                else self.registered_checks
+            )
             checks.add(check)
             return check
 
@@ -56,7 +68,13 @@ class CheckRegistry:
                 tags += (check,)
             return inner
 
-    def run_checks(self, app_configs=None, tags=None, include_deployment_checks=False, databases=None):
+    def run_checks(
+        self,
+        app_configs=None,
+        tags=None,
+        include_deployment_checks=False,
+        databases=None,
+    ):
         """
         Run all registered checks and return list of Errors and Warnings.
         """
@@ -68,9 +86,11 @@ class CheckRegistry:
 
         for check in checks:
             new_errors = check(app_configs=app_configs, databases=databases)
-            assert is_iterable(new_errors), (
-                "The function %r did not return a list. All functions registered "
-                "with the checks registry must return a list." % check)
+            if not is_iterable(new_errors):
+                raise TypeError(
+                    "The function %r did not return a list. All functions "
+                    "registered with the checks registry must return a list." % check,
+                )
             errors.extend(new_errors)
         return errors
 
@@ -78,9 +98,11 @@ class CheckRegistry:
         return tag in self.tags_available(include_deployment_checks)
 
     def tags_available(self, deployment_checks=False):
-        return set(chain.from_iterable(
-            check.tags for check in self.get_checks(deployment_checks)
-        ))
+        return set(
+            chain.from_iterable(
+                check.tags for check in self.get_checks(deployment_checks)
+            )
+        )
 
     def get_checks(self, include_deployment_checks=False):
         checks = list(self.registered_checks)
