@@ -19,8 +19,11 @@ from django.utils.module_loading import import_string
 from django.utils.text import get_valid_filename
 
 __all__ = (
-    'Storage', 'FileSystemStorage', 'DefaultStorage', 'default_storage',
-    'get_storage_class',
+    "Storage",
+    "FileSystemStorage",
+    "DefaultStorage",
+    "default_storage",
+    "get_storage_class",
 )
 
 
@@ -33,7 +36,7 @@ class Storage:
     # The following methods represent a public interface to private methods.
     # These shouldn't be overridden by subclasses unless absolutely necessary.
 
-    def open(self, name, mode='rb'):
+    def open(self, name, mode="rb"):
         """Retrieve the specified file from storage."""
         return self._open(name, mode)
 
@@ -47,11 +50,14 @@ class Storage:
         if name is None:
             name = content.name
 
-        if not hasattr(content, 'chunks'):
+        if not hasattr(content, "chunks"):
             content = File(content, name)
 
         name = self.get_available_name(name, max_length=max_length)
-        return self._save(name, content)
+        name = self._save(name, content)
+        # Ensure that the name returned from the storage system is still valid.
+        validate_file_name(name, allow_relative_path=True)
+        return name
 
     # These methods are part of the public API, with default implementations.
 
@@ -68,16 +74,19 @@ class Storage:
         character alphanumeric string (before the file extension, if one
         exists) to the filename.
         """
-        return '%s_%s%s' % (file_root, get_random_string(7), file_ext)
+        return "%s_%s%s" % (file_root, get_random_string(7), file_ext)
 
     def get_available_name(self, name, max_length=None):
         """
         Return a filename that's free on the target storage system and
         available for new content to be written to.
         """
+        name = str(name).replace("\\", "/")
         dir_name, file_name = os.path.split(name)
-        if '..' in pathlib.PurePath(dir_name).parts:
-            raise SuspiciousFileOperation("Detected path traversal attempt in '%s'" % dir_name)
+        if ".." in pathlib.PurePath(dir_name).parts:
+            raise SuspiciousFileOperation(
+                "Detected path traversal attempt in '%s'" % dir_name
+            )
         validate_file_name(file_name)
         file_root, file_ext = os.path.splitext(file_name)
         # If the filename already exists, generate an alternative filename
@@ -86,21 +95,26 @@ class Storage:
         # exceed the max_length.
         while self.exists(name) or (max_length and len(name) > max_length):
             # file_ext includes the dot.
-            name = os.path.join(dir_name, self.get_alternative_name(file_root, file_ext))
+            name = os.path.join(
+                dir_name, self.get_alternative_name(file_root, file_ext)
+            )
             if max_length is None:
                 continue
             # Truncate file_root if max_length exceeded.
             truncation = len(name) - max_length
             if truncation > 0:
                 file_root = file_root[:-truncation]
-                # Entire file_root was truncated in attempt to find an available filename.
+                # Entire file_root was truncated in attempt to find an
+                # available filename.
                 if not file_root:
                     raise SuspiciousFileOperation(
                         'Storage can not find an available filename for "%s". '
-                        'Please make sure that the corresponding file field '
+                        "Please make sure that the corresponding file field "
                         'allows sufficient "max_length".' % name
                     )
-                name = os.path.join(dir_name, self.get_alternative_name(file_root, file_ext))
+                name = os.path.join(
+                    dir_name, self.get_alternative_name(file_root, file_ext)
+                )
         return name
 
     def generate_filename(self, filename):
@@ -108,10 +122,13 @@ class Storage:
         Validate the filename by calling get_valid_name() and return a filename
         to be passed to the save() method.
         """
+        filename = str(filename).replace("\\", "/")
         # `filename` may include a path as returned by FileField.upload_to.
         dirname, filename = os.path.split(filename)
-        if '..' in pathlib.PurePath(dirname).parts:
-            raise SuspiciousFileOperation("Detected path traversal attempt in '%s'" % dirname)
+        if ".." in pathlib.PurePath(dirname).parts:
+            raise SuspiciousFileOperation(
+                "Detected path traversal attempt in '%s'" % dirname
+            )
         return os.path.normpath(os.path.join(dirname, self.get_valid_name(filename)))
 
     def path(self, name):
@@ -129,55 +146,67 @@ class Storage:
         """
         Delete the specified file from the storage system.
         """
-        raise NotImplementedError('subclasses of Storage must provide a delete() method')
+        raise NotImplementedError(
+            "subclasses of Storage must provide a delete() method"
+        )
 
     def exists(self, name):
         """
         Return True if a file referenced by the given name already exists in the
         storage system, or False if the name is available for a new file.
         """
-        raise NotImplementedError('subclasses of Storage must provide an exists() method')
+        raise NotImplementedError(
+            "subclasses of Storage must provide an exists() method"
+        )
 
     def listdir(self, path):
         """
         List the contents of the specified path. Return a 2-tuple of lists:
         the first item being directories, the second item being files.
         """
-        raise NotImplementedError('subclasses of Storage must provide a listdir() method')
+        raise NotImplementedError(
+            "subclasses of Storage must provide a listdir() method"
+        )
 
     def size(self, name):
         """
         Return the total size, in bytes, of the file specified by name.
         """
-        raise NotImplementedError('subclasses of Storage must provide a size() method')
+        raise NotImplementedError("subclasses of Storage must provide a size() method")
 
     def url(self, name):
         """
         Return an absolute URL where the file's contents can be accessed
-        directly by a Web browser.
+        directly by a web browser.
         """
-        raise NotImplementedError('subclasses of Storage must provide a url() method')
+        raise NotImplementedError("subclasses of Storage must provide a url() method")
 
     def get_accessed_time(self, name):
         """
         Return the last accessed time (as a datetime) of the file specified by
         name. The datetime will be timezone-aware if USE_TZ=True.
         """
-        raise NotImplementedError('subclasses of Storage must provide a get_accessed_time() method')
+        raise NotImplementedError(
+            "subclasses of Storage must provide a get_accessed_time() method"
+        )
 
     def get_created_time(self, name):
         """
         Return the creation time (as a datetime) of the file specified by name.
         The datetime will be timezone-aware if USE_TZ=True.
         """
-        raise NotImplementedError('subclasses of Storage must provide a get_created_time() method')
+        raise NotImplementedError(
+            "subclasses of Storage must provide a get_created_time() method"
+        )
 
     def get_modified_time(self, name):
         """
         Return the last modified time (as a datetime) of the file specified by
         name. The datetime will be timezone-aware if USE_TZ=True.
         """
-        raise NotImplementedError('subclasses of Storage must provide a get_modified_time() method')
+        raise NotImplementedError(
+            "subclasses of Storage must provide a get_modified_time() method"
+        )
 
 
 @deconstructible
@@ -185,12 +214,18 @@ class FileSystemStorage(Storage):
     """
     Standard filesystem storage
     """
+
     # The combination of O_CREAT and O_EXCL makes os.open() raise OSError if
     # the file already exists before it's opened.
-    OS_OPEN_FLAGS = os.O_WRONLY | os.O_CREAT | os.O_EXCL | getattr(os, 'O_BINARY', 0)
+    OS_OPEN_FLAGS = os.O_WRONLY | os.O_CREAT | os.O_EXCL | getattr(os, "O_BINARY", 0)
 
-    def __init__(self, location=None, base_url=None, file_permissions_mode=None,
-                 directory_permissions_mode=None):
+    def __init__(
+        self,
+        location=None,
+        base_url=None,
+        file_permissions_mode=None,
+        directory_permissions_mode=None,
+    ):
         self._location = location
         self._base_url = base_url
         self._file_permissions_mode = file_permissions_mode
@@ -199,15 +234,15 @@ class FileSystemStorage(Storage):
 
     def _clear_cached_properties(self, setting, **kwargs):
         """Reset setting based property values."""
-        if setting == 'MEDIA_ROOT':
-            self.__dict__.pop('base_location', None)
-            self.__dict__.pop('location', None)
-        elif setting == 'MEDIA_URL':
-            self.__dict__.pop('base_url', None)
-        elif setting == 'FILE_UPLOAD_PERMISSIONS':
-            self.__dict__.pop('file_permissions_mode', None)
-        elif setting == 'FILE_UPLOAD_DIRECTORY_PERMISSIONS':
-            self.__dict__.pop('directory_permissions_mode', None)
+        if setting == "MEDIA_ROOT":
+            self.__dict__.pop("base_location", None)
+            self.__dict__.pop("location", None)
+        elif setting == "MEDIA_URL":
+            self.__dict__.pop("base_url", None)
+        elif setting == "FILE_UPLOAD_PERMISSIONS":
+            self.__dict__.pop("file_permissions_mode", None)
+        elif setting == "FILE_UPLOAD_DIRECTORY_PERMISSIONS":
+            self.__dict__.pop("directory_permissions_mode", None)
 
     def _value_or_setting(self, value, setting):
         return setting if value is None else value
@@ -222,19 +257,23 @@ class FileSystemStorage(Storage):
 
     @cached_property
     def base_url(self):
-        if self._base_url is not None and not self._base_url.endswith('/'):
-            self._base_url += '/'
+        if self._base_url is not None and not self._base_url.endswith("/"):
+            self._base_url += "/"
         return self._value_or_setting(self._base_url, settings.MEDIA_URL)
 
     @cached_property
     def file_permissions_mode(self):
-        return self._value_or_setting(self._file_permissions_mode, settings.FILE_UPLOAD_PERMISSIONS)
+        return self._value_or_setting(
+            self._file_permissions_mode, settings.FILE_UPLOAD_PERMISSIONS
+        )
 
     @cached_property
     def directory_permissions_mode(self):
-        return self._value_or_setting(self._directory_permissions_mode, settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS)
+        return self._value_or_setting(
+            self._directory_permissions_mode, settings.FILE_UPLOAD_DIRECTORY_PERMISSIONS
+        )
 
-    def _open(self, name, mode='rb'):
+    def _open(self, name, mode="rb"):
         return File(open(self.path(name), mode))
 
     def _save(self, name, content):
@@ -248,13 +287,15 @@ class FileSystemStorage(Storage):
                 # argument to intermediate-level directories.
                 old_umask = os.umask(0o777 & ~self.directory_permissions_mode)
                 try:
-                    os.makedirs(directory, self.directory_permissions_mode, exist_ok=True)
+                    os.makedirs(
+                        directory, self.directory_permissions_mode, exist_ok=True
+                    )
                 finally:
                     os.umask(old_umask)
             else:
                 os.makedirs(directory, exist_ok=True)
         except FileExistsError:
-            raise FileExistsError('%s exists and is not a directory.' % directory)
+            raise FileExistsError("%s exists and is not a directory." % directory)
 
         # There's a potential race condition between get_available_name and
         # saving the file; it's possible that two threads might return the
@@ -265,7 +306,7 @@ class FileSystemStorage(Storage):
         while True:
             try:
                 # This file has a file path that we can move.
-                if hasattr(content, 'temporary_file_path'):
+                if hasattr(content, "temporary_file_path"):
                     file_move_safe(content.temporary_file_path(), full_path)
 
                 # This is a normal uploadedfile that we can stream.
@@ -277,7 +318,7 @@ class FileSystemStorage(Storage):
                         locks.lock(fd, locks.LOCK_EX)
                         for chunk in content.chunks():
                             if _file is None:
-                                mode = 'wb' if isinstance(chunk, bytes) else 'wt'
+                                mode = "wb" if isinstance(chunk, bytes) else "wt"
                                 _file = os.fdopen(fd, mode)
                             _file.write(chunk)
                     finally:
@@ -297,11 +338,14 @@ class FileSystemStorage(Storage):
         if self.file_permissions_mode is not None:
             os.chmod(full_path, self.file_permissions_mode)
 
+        # Ensure the saved path is always relative to the storage root.
+        name = os.path.relpath(full_path, self.location)
         # Store filenames with forward slashes, even on Windows.
-        return str(name).replace('\\', '/')
+        return str(name).replace("\\", "/")
 
     def delete(self, name):
-        assert name, "The name argument is not allowed to be empty."
+        if not name:
+            raise ValueError("The name must be given to delete().")
         name = self.path(name)
         # If the file or directory exists, delete it from the filesystem.
         try:
@@ -315,16 +359,17 @@ class FileSystemStorage(Storage):
             pass
 
     def exists(self, name):
-        return os.path.exists(self.path(name))
+        return os.path.lexists(self.path(name))
 
     def listdir(self, path):
         path = self.path(path)
         directories, files = [], []
-        for entry in os.scandir(path):
-            if entry.is_dir():
-                directories.append(entry.name)
-            else:
-                files.append(entry.name)
+        with os.scandir(path) as entries:
+            for entry in entries:
+                if entry.is_dir():
+                    directories.append(entry.name)
+                else:
+                    files.append(entry.name)
         return directories, files
 
     def path(self, name):
@@ -338,7 +383,7 @@ class FileSystemStorage(Storage):
             raise ValueError("This file is not accessible via a URL.")
         url = filepath_to_uri(name)
         if url is not None:
-            url = url.lstrip('/')
+            url = url.lstrip("/")
         return urljoin(self.base_url, url)
 
     def _datetime_from_timestamp(self, ts):
@@ -346,11 +391,8 @@ class FileSystemStorage(Storage):
         If timezone support is enabled, make an aware datetime object in UTC;
         otherwise make a naive one in the local timezone.
         """
-        if settings.USE_TZ:
-            # Safe to use .replace() because UTC doesn't have DST
-            return datetime.utcfromtimestamp(ts).replace(tzinfo=timezone.utc)
-        else:
-            return datetime.fromtimestamp(ts)
+        tz = timezone.utc if settings.USE_TZ else None
+        return datetime.fromtimestamp(ts, tz=tz)
 
     def get_accessed_time(self, name):
         return self._datetime_from_timestamp(os.path.getatime(self.path(name)))
