@@ -8,7 +8,10 @@ from asgiref.sync import sync_to_async
 from django.conf import settings
 from django.core import signals
 from django.core.exceptions import (
-    PermissionDenied, RequestDataTooBig, SuspiciousOperation,
+    BadRequest,
+    PermissionDenied,
+    RequestDataTooBig,
+    SuspiciousOperation,
     TooManyFieldsSent,
 )
 from django.http import Http404
@@ -32,15 +35,20 @@ def convert_exception_to_response(get_response):
     can rely on getting a response instead of an exception.
     """
     if asyncio.iscoroutinefunction(get_response):
+
         @wraps(get_response)
         async def inner(request):
             try:
                 response = await get_response(request)
             except Exception as exc:
-                response = await sync_to_async(response_for_exception)(request, exc)
+                response = await sync_to_async(
+                    response_for_exception, thread_sensitive=False
+                )(request, exc)
             return response
+
         return inner
     else:
+
         @wraps(get_response)
         def inner(request):
             try:
@@ -48,6 +56,7 @@ def convert_exception_to_response(get_response):
             except Exception as exc:
                 response = response_for_exception(request, exc)
             return response
+
         return inner
 
 
@@ -56,26 +65,51 @@ def response_for_exception(request, exc):
         if settings.DEBUG:
             response = debug.technical_404_response(request, exc)
         else:
-            response = get_exception_response(request, get_resolver(get_urlconf()), 404, exc)
+            response = get_exception_response(
+                request, get_resolver(get_urlconf()), 404, exc
+            )
 
     elif isinstance(exc, PermissionDenied):
-        response = get_exception_response(request, get_resolver(get_urlconf()), 403, exc)
+        response = get_exception_response(
+            request, get_resolver(get_urlconf()), 403, exc
+        )
         log_response(
-            'Forbidden (Permission denied): %s', request.path,
+            "Forbidden (Permission denied): %s",
+            request.path,
             response=response,
             request=request,
             exc_info=sys.exc_info(),
         )
 
     elif isinstance(exc, MultiPartParserError):
-        response = get_exception_response(request, get_resolver(get_urlconf()), 400, exc)
+        response = get_exception_response(
+            request, get_resolver(get_urlconf()), 400, exc
+        )
         log_response(
-            'Bad request (Unable to parse request body): %s', request.path,
+            "Bad request (Unable to parse request body): %s",
+            request.path,
             response=response,
             request=request,
             exc_info=sys.exc_info(),
         )
 
+    elif isinstance(exc, BadRequest):
+        if settings.DEBUG:
+            response = debug.technical_500_response(
+                request, *sys.exc_info(), status_code=400
+            )
+        else:
+            response = get_exception_response(
+                request, get_resolver(get_urlconf()), 400, exc
+            )
+        log_response(
+            "%s: %s",
+            str(exc),
+            request.path,
+            response=response,
+            request=request,
+            exc_info=sys.exc_info(),
+        )
     elif isinstance(exc, SuspiciousOperation):
         if isinstance(exc, (RequestDataTooBig, TooManyFieldsSent)):
             # POST data can't be accessed again, otherwise the original
@@ -84,32 +118,40 @@ def response_for_exception(request, exc):
 
         # The request logger receives events for any problematic request
         # The security logger receives events for all SuspiciousOperations
-        security_logger = logging.getLogger('django.security.%s' % exc.__class__.__name__)
+        security_logger = logging.getLogger(
+            "django.security.%s" % exc.__class__.__name__
+        )
         security_logger.error(
             str(exc),
-            extra={'status_code': 400, 'request': request},
+            extra={"status_code": 400, "request": request},
         )
         if settings.DEBUG:
-            response = debug.technical_500_response(request, *sys.exc_info(), status_code=400)
+            response = debug.technical_500_response(
+                request, *sys.exc_info(), status_code=400
+            )
         else:
-            response = get_exception_response(request, get_resolver(get_urlconf()), 400, exc)
-
-    elif isinstance(exc, SystemExit):
-        # Allow sys.exit() to actually exit. See tickets #1023 and #4701
-        raise
+            response = get_exception_response(
+                request, get_resolver(get_urlconf()), 400, exc
+            )
 
     else:
         signals.got_request_exception.send(sender=None, request=request)
-        response = handle_uncaught_exception(request, get_resolver(get_urlconf()), sys.exc_info())
+        response = handle_uncaught_exception(
+            request, get_resolver(get_urlconf()), sys.exc_info()
+        )
         log_response(
-            '%s: %s', response.reason_phrase, request.path,
+            "%s: %s",
+            response.reason_phrase,
+            request.path,
             response=response,
             request=request,
             exc_info=sys.exc_info(),
         )
 
     # Force a TemplateResponse to be rendered.
-    if not getattr(response, 'is_rendered', True) and callable(getattr(response, 'render', None)):
+    if not getattr(response, "is_rendered", True) and callable(
+        getattr(response, "render", None)
+    ):
         response = response.render()
 
     return response
@@ -117,8 +159,8 @@ def response_for_exception(request, exc):
 
 def get_exception_response(request, resolver, status_code, exception):
     try:
-        callback, param_dict = resolver.resolve_error_handler(status_code)
-        response = callback(request, **{**param_dict, 'exception': exception})
+        callback = resolver.resolve_error_handler(status_code)
+        response = callback(request, exception=exception)
     except Exception:
         signals.got_request_exception.send(sender=None, request=request)
         response = handle_uncaught_exception(request, resolver, sys.exc_info())
@@ -138,5 +180,5 @@ def handle_uncaught_exception(request, resolver, exc_info):
         return debug.technical_500_response(request, *exc_info)
 
     # Return an HttpResponse that displays a friendly error message.
-    callback, param_dict = resolver.resolve_error_handler(500)
-    return callback(request, **param_dict)
+    callback = resolver.resolve_error_handler(500)
+    return callback(request)
