@@ -21,27 +21,31 @@ from django.utils.functional import cached_property
 def _setup_environment(environ):
     # Cygwin requires some special voodoo to set the environment variables
     # properly so that Oracle will see them.
-    if platform.system().upper().startswith('CYGWIN'):
+    if platform.system().upper().startswith("CYGWIN"):
         try:
             import ctypes
         except ImportError as e:
-            raise ImproperlyConfigured("Error loading ctypes: %s; "
-                                       "the Oracle backend requires ctypes to "
-                                       "operate correctly under Cygwin." % e)
-        kernel32 = ctypes.CDLL('kernel32')
+            raise ImproperlyConfigured(
+                "Error loading ctypes: %s; "
+                "the Oracle backend requires ctypes to "
+                "operate correctly under Cygwin." % e
+            )
+        kernel32 = ctypes.CDLL("kernel32")
         for name, value in environ:
             kernel32.SetEnvironmentVariableA(name, value)
     else:
         os.environ.update(environ)
 
 
-_setup_environment([
-    # Oracle takes client-side character set encoding from the environment.
-    ('NLS_LANG', '.AL32UTF8'),
-    # This prevents Unicode from getting mangled by getting encoded into the
-    # potentially non-Unicode database character set.
-    ('ORA_NCHAR_LITERAL_REPLACE', 'TRUE'),
-])
+_setup_environment(
+    [
+        # Oracle takes client-side character set encoding from the environment.
+        ("NLS_LANG", ".AL32UTF8"),
+        # This prevents Unicode from getting mangled by getting encoded into the
+        # potentially non-Unicode database character set.
+        ("ORA_NCHAR_LITERAL_REPLACE", "TRUE"),
+    ]
+)
 
 
 try:
@@ -50,14 +54,14 @@ except ImportError as e:
     raise ImproperlyConfigured("Error loading cx_Oracle module: %s" % e)
 
 # Some of these import cx_Oracle, so import them after checking if it's installed.
-from .client import DatabaseClient                          # NOQA isort:skip
-from .creation import DatabaseCreation                      # NOQA isort:skip
-from .features import DatabaseFeatures                      # NOQA isort:skip
-from .introspection import DatabaseIntrospection            # NOQA isort:skip
-from .operations import DatabaseOperations                  # NOQA isort:skip
-from .schema import DatabaseSchemaEditor                    # NOQA isort:skip
-from .utils import Oracle_datetime                          # NOQA isort:skip
-from .validation import DatabaseValidation                  # NOQA isort:skip
+from .client import DatabaseClient  # NOQA
+from .creation import DatabaseCreation  # NOQA
+from .features import DatabaseFeatures  # NOQA
+from .introspection import DatabaseIntrospection  # NOQA
+from .operations import DatabaseOperations  # NOQA
+from .schema import DatabaseSchemaEditor  # NOQA
+from .utils import Oracle_datetime, dsn  # NOQA
+from .validation import DatabaseValidation  # NOQA
 
 
 @contextmanager
@@ -77,17 +81,16 @@ def wrap_oracle_errors():
         # Convert that case to Django's IntegrityError exception.
         x = e.args[0]
         if (
-            hasattr(x, 'code') and
-            hasattr(x, 'message') and
-            x.code == 2091 and
-            ('ORA-02291' in x.message or 'ORA-00001' in x.message)
+            hasattr(x, "code")
+            and hasattr(x, "message")
+            and x.code == 2091
+            and ("ORA-02291" in x.message or "ORA-00001" in x.message)
         ):
             raise IntegrityError(*tuple(e.args))
         raise
 
 
 class _UninitializedOperatorsDescriptor:
-
     def __get__(self, instance, cls=None):
         # If connection.operators is looked up before a connection has been
         # created, transparently initialize connection.operators to avert an
@@ -96,12 +99,12 @@ class _UninitializedOperatorsDescriptor:
             raise AttributeError("operators not available as class attribute")
         # Creating a cursor will initialize the operators.
         instance.cursor().close()
-        return instance.__dict__['operators']
+        return instance.__dict__["operators"]
 
 
 class DatabaseWrapper(BaseDatabaseWrapper):
-    vendor = 'oracle'
-    display_name = 'Oracle'
+    vendor = "oracle"
+    display_name = "Oracle"
     # This dictionary maps Field objects to their associated Oracle column
     # types, as strings. Column-type strings can contain format strings; they'll
     # be interpolated against the values of Field.__dict__ before being output.
@@ -110,73 +113,86 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     # Any format strings starting with "qn_" are quoted before being used in the
     # output (the "qn_" prefix is stripped before the lookup is performed.
     data_types = {
-        'AutoField': 'NUMBER(11) GENERATED BY DEFAULT ON NULL AS IDENTITY',
-        'BigAutoField': 'NUMBER(19) GENERATED BY DEFAULT ON NULL AS IDENTITY',
-        'BinaryField': 'BLOB',
-        'BooleanField': 'NUMBER(1)',
-        'CharField': 'NVARCHAR2(%(max_length)s)',
-        'DateField': 'DATE',
-        'DateTimeField': 'TIMESTAMP',
-        'DecimalField': 'NUMBER(%(max_digits)s, %(decimal_places)s)',
-        'DurationField': 'INTERVAL DAY(9) TO SECOND(6)',
-        'FileField': 'NVARCHAR2(%(max_length)s)',
-        'FilePathField': 'NVARCHAR2(%(max_length)s)',
-        'FloatField': 'DOUBLE PRECISION',
-        'IntegerField': 'NUMBER(11)',
-        'JSONField': 'NCLOB',
-        'BigIntegerField': 'NUMBER(19)',
-        'IPAddressField': 'VARCHAR2(15)',
-        'GenericIPAddressField': 'VARCHAR2(39)',
-        'NullBooleanField': 'NUMBER(1)',
-        'OneToOneField': 'NUMBER(11)',
-        'PositiveBigIntegerField': 'NUMBER(19)',
-        'PositiveIntegerField': 'NUMBER(11)',
-        'PositiveSmallIntegerField': 'NUMBER(11)',
-        'SlugField': 'NVARCHAR2(%(max_length)s)',
-        'SmallAutoField': 'NUMBER(5) GENERATED BY DEFAULT ON NULL AS IDENTITY',
-        'SmallIntegerField': 'NUMBER(11)',
-        'TextField': 'NCLOB',
-        'TimeField': 'TIMESTAMP',
-        'URLField': 'VARCHAR2(%(max_length)s)',
-        'UUIDField': 'VARCHAR2(32)',
+        "AutoField": "NUMBER(11) GENERATED BY DEFAULT ON NULL AS IDENTITY",
+        "BigAutoField": "NUMBER(19) GENERATED BY DEFAULT ON NULL AS IDENTITY",
+        "BinaryField": "BLOB",
+        "BooleanField": "NUMBER(1)",
+        "CharField": "NVARCHAR2(%(max_length)s)",
+        "DateField": "DATE",
+        "DateTimeField": "TIMESTAMP",
+        "DecimalField": "NUMBER(%(max_digits)s, %(decimal_places)s)",
+        "DurationField": "INTERVAL DAY(9) TO SECOND(6)",
+        "FileField": "NVARCHAR2(%(max_length)s)",
+        "FilePathField": "NVARCHAR2(%(max_length)s)",
+        "FloatField": "DOUBLE PRECISION",
+        "IntegerField": "NUMBER(11)",
+        "JSONField": "NCLOB",
+        "BigIntegerField": "NUMBER(19)",
+        "IPAddressField": "VARCHAR2(15)",
+        "GenericIPAddressField": "VARCHAR2(39)",
+        "OneToOneField": "NUMBER(11)",
+        "PositiveBigIntegerField": "NUMBER(19)",
+        "PositiveIntegerField": "NUMBER(11)",
+        "PositiveSmallIntegerField": "NUMBER(11)",
+        "SlugField": "NVARCHAR2(%(max_length)s)",
+        "SmallAutoField": "NUMBER(5) GENERATED BY DEFAULT ON NULL AS IDENTITY",
+        "SmallIntegerField": "NUMBER(11)",
+        "TextField": "NCLOB",
+        "TimeField": "TIMESTAMP",
+        "URLField": "VARCHAR2(%(max_length)s)",
+        "UUIDField": "VARCHAR2(32)",
     }
     data_type_check_constraints = {
-        'BooleanField': '%(qn_column)s IN (0,1)',
-        'JSONField': '%(qn_column)s IS JSON',
-        'NullBooleanField': '%(qn_column)s IN (0,1)',
-        'PositiveBigIntegerField': '%(qn_column)s >= 0',
-        'PositiveIntegerField': '%(qn_column)s >= 0',
-        'PositiveSmallIntegerField': '%(qn_column)s >= 0',
+        "BooleanField": "%(qn_column)s IN (0,1)",
+        "JSONField": "%(qn_column)s IS JSON",
+        "PositiveBigIntegerField": "%(qn_column)s >= 0",
+        "PositiveIntegerField": "%(qn_column)s >= 0",
+        "PositiveSmallIntegerField": "%(qn_column)s >= 0",
     }
 
     # Oracle doesn't support a database index on these columns.
-    _limited_data_types = ('clob', 'nclob', 'blob')
+    _limited_data_types = ("clob", "nclob", "blob")
 
     operators = _UninitializedOperatorsDescriptor()
 
     _standard_operators = {
-        'exact': '= %s',
-        'iexact': '= UPPER(%s)',
-        'contains': "LIKE TRANSLATE(%s USING NCHAR_CS) ESCAPE TRANSLATE('\\' USING NCHAR_CS)",
-        'icontains': "LIKE UPPER(TRANSLATE(%s USING NCHAR_CS)) ESCAPE TRANSLATE('\\' USING NCHAR_CS)",
-        'gt': '> %s',
-        'gte': '>= %s',
-        'lt': '< %s',
-        'lte': '<= %s',
-        'startswith': "LIKE TRANSLATE(%s USING NCHAR_CS) ESCAPE TRANSLATE('\\' USING NCHAR_CS)",
-        'endswith': "LIKE TRANSLATE(%s USING NCHAR_CS) ESCAPE TRANSLATE('\\' USING NCHAR_CS)",
-        'istartswith': "LIKE UPPER(TRANSLATE(%s USING NCHAR_CS)) ESCAPE TRANSLATE('\\' USING NCHAR_CS)",
-        'iendswith': "LIKE UPPER(TRANSLATE(%s USING NCHAR_CS)) ESCAPE TRANSLATE('\\' USING NCHAR_CS)",
+        "exact": "= %s",
+        "iexact": "= UPPER(%s)",
+        "contains": (
+            "LIKE TRANSLATE(%s USING NCHAR_CS) ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        ),
+        "icontains": (
+            "LIKE UPPER(TRANSLATE(%s USING NCHAR_CS)) "
+            "ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        ),
+        "gt": "> %s",
+        "gte": ">= %s",
+        "lt": "< %s",
+        "lte": "<= %s",
+        "startswith": (
+            "LIKE TRANSLATE(%s USING NCHAR_CS) ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        ),
+        "endswith": (
+            "LIKE TRANSLATE(%s USING NCHAR_CS) ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        ),
+        "istartswith": (
+            "LIKE UPPER(TRANSLATE(%s USING NCHAR_CS)) "
+            "ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        ),
+        "iendswith": (
+            "LIKE UPPER(TRANSLATE(%s USING NCHAR_CS)) "
+            "ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        ),
     }
 
     _likec_operators = {
         **_standard_operators,
-        'contains': "LIKEC %s ESCAPE '\\'",
-        'icontains': "LIKEC UPPER(%s) ESCAPE '\\'",
-        'startswith': "LIKEC %s ESCAPE '\\'",
-        'endswith': "LIKEC %s ESCAPE '\\'",
-        'istartswith': "LIKEC UPPER(%s) ESCAPE '\\'",
-        'iendswith': "LIKEC UPPER(%s) ESCAPE '\\'",
+        "contains": "LIKEC %s ESCAPE '\\'",
+        "icontains": "LIKEC UPPER(%s) ESCAPE '\\'",
+        "startswith": "LIKEC %s ESCAPE '\\'",
+        "endswith": "LIKEC %s ESCAPE '\\'",
+        "istartswith": "LIKEC UPPER(%s) ESCAPE '\\'",
+        "iendswith": "LIKEC UPPER(%s) ESCAPE '\\'",
     }
 
     # The patterns below are used to generate SQL pattern lookup clauses when
@@ -189,19 +205,22 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     # the LIKE operator.
     pattern_esc = r"REPLACE(REPLACE(REPLACE({}, '\', '\\'), '%%', '\%%'), '_', '\_')"
     _pattern_ops = {
-        'contains': "'%%' || {} || '%%'",
-        'icontains': "'%%' || UPPER({}) || '%%'",
-        'startswith': "{} || '%%'",
-        'istartswith': "UPPER({}) || '%%'",
-        'endswith': "'%%' || {}",
-        'iendswith': "'%%' || UPPER({})",
+        "contains": "'%%' || {} || '%%'",
+        "icontains": "'%%' || UPPER({}) || '%%'",
+        "startswith": "{} || '%%'",
+        "istartswith": "UPPER({}) || '%%'",
+        "endswith": "'%%' || {}",
+        "iendswith": "'%%' || UPPER({})",
     }
 
-    _standard_pattern_ops = {k: "LIKE TRANSLATE( " + v + " USING NCHAR_CS)"
-                                " ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
-                             for k, v in _pattern_ops.items()}
-    _likec_pattern_ops = {k: "LIKEC " + v + " ESCAPE '\\'"
-                          for k, v in _pattern_ops.items()}
+    _standard_pattern_ops = {
+        k: "LIKE TRANSLATE( " + v + " USING NCHAR_CS)"
+        " ESCAPE TRANSLATE('\\' USING NCHAR_CS)"
+        for k, v in _pattern_ops.items()
+    }
+    _likec_pattern_ops = {
+        k: "LIKEC " + v + " ESCAPE '\\'" for k, v in _pattern_ops.items()
+    }
 
     Database = Database
     SchemaEditorClass = DatabaseSchemaEditor
@@ -215,32 +234,23 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
-        use_returning_into = self.settings_dict["OPTIONS"].get('use_returning_into', True)
+        use_returning_into = self.settings_dict["OPTIONS"].get(
+            "use_returning_into", True
+        )
         self.features.can_return_columns_from_insert = use_returning_into
 
-    def _dsn(self):
-        settings_dict = self.settings_dict
-        if not settings_dict['HOST'].strip():
-            settings_dict['HOST'] = 'localhost'
-        if settings_dict['PORT']:
-            return Database.makedsn(settings_dict['HOST'], int(settings_dict['PORT']), settings_dict['NAME'])
-        return settings_dict['NAME']
-
-    def _connect_string(self):
-        return '%s/"%s"@%s' % (self.settings_dict['USER'], self.settings_dict['PASSWORD'], self._dsn())
-
     def get_connection_params(self):
-        conn_params = self.settings_dict['OPTIONS'].copy()
-        if 'use_returning_into' in conn_params:
-            del conn_params['use_returning_into']
+        conn_params = self.settings_dict["OPTIONS"].copy()
+        if "use_returning_into" in conn_params:
+            del conn_params["use_returning_into"]
         return conn_params
 
     @async_unsafe
     def get_new_connection(self, conn_params):
         return Database.connect(
-            user=self.settings_dict['USER'],
-            password=self.settings_dict['PASSWORD'],
-            dsn=self._dsn(),
+            user=self.settings_dict["USER"],
+            password=self.settings_dict["PASSWORD"],
+            dsn=dsn(self.settings_dict),
             **conn_params,
         )
 
@@ -257,11 +267,11 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         # TO_CHAR().
         cursor.execute(
             "ALTER SESSION SET NLS_DATE_FORMAT = 'YYYY-MM-DD HH24:MI:SS'"
-            " NLS_TIMESTAMP_FORMAT = 'YYYY-MM-DD HH24:MI:SS.FF'" +
-            (" TIME_ZONE = 'UTC'" if settings.USE_TZ else '')
+            " NLS_TIMESTAMP_FORMAT = 'YYYY-MM-DD HH24:MI:SS.FF'"
+            + (" TIME_ZONE = 'UTC'" if settings.USE_TZ else "")
         )
         cursor.close()
-        if 'operators' not in self.__dict__:
+        if "operators" not in self.__dict__:
             # Ticket #14149: Check whether our LIKE implementation will
             # work for this connection or we need to fall back on LIKEC.
             # This check is performed only once per DatabaseWrapper
@@ -269,9 +279,11 @@ class DatabaseWrapper(BaseDatabaseWrapper):
             # the same settings.
             cursor = self.create_cursor()
             try:
-                cursor.execute("SELECT 1 FROM DUAL WHERE DUMMY %s"
-                               % self._standard_operators['contains'],
-                               ['X'])
+                cursor.execute(
+                    "SELECT 1 FROM DUAL WHERE DUMMY %s"
+                    % self._standard_operators["contains"],
+                    ["X"],
+                )
             except Database.DatabaseError:
                 self.operators = self._likec_operators
                 self.pattern_ops = self._likec_pattern_ops
@@ -297,10 +309,12 @@ class DatabaseWrapper(BaseDatabaseWrapper):
     # logging is enabled to keep query counts consistent with other backends.
     def _savepoint_commit(self, sid):
         if self.queries_logged:
-            self.queries_log.append({
-                'sql': '-- RELEASE SAVEPOINT %s (faked)' % self.ops.quote_name(sid),
-                'time': '0.000',
-            })
+            self.queries_log.append(
+                {
+                    "sql": "-- RELEASE SAVEPOINT %s (faked)" % self.ops.quote_name(sid),
+                    "time": "0.000",
+                }
+            )
 
     def _set_autocommit(self, autocommit):
         with self.wrap_database_errors:
@@ -312,8 +326,8 @@ class DatabaseWrapper(BaseDatabaseWrapper):
         afterward.
         """
         with self.cursor() as cursor:
-            cursor.execute('SET CONSTRAINTS ALL IMMEDIATE')
-            cursor.execute('SET CONSTRAINTS ALL DEFERRED')
+            cursor.execute("SET CONSTRAINTS ALL IMMEDIATE")
+            cursor.execute("SET CONSTRAINTS ALL DEFERRED")
 
     def is_usable(self):
         try:
@@ -325,12 +339,12 @@ class DatabaseWrapper(BaseDatabaseWrapper):
 
     @cached_property
     def cx_oracle_version(self):
-        return tuple(int(x) for x in Database.version.split('.'))
+        return tuple(int(x) for x in Database.version.split("."))
 
     @cached_property
     def oracle_version(self):
         with self.temporary_connection():
-            return tuple(int(x) for x in self.connection.version.split('.'))
+            return tuple(int(x) for x in self.connection.version.split("."))
 
 
 class OracleParam:
@@ -346,8 +360,10 @@ class OracleParam:
     def __init__(self, param, cursor, strings_only=False):
         # With raw SQL queries, datetimes can reach this function
         # without being converted by DateTimeField.get_db_prep_value.
-        if settings.USE_TZ and (isinstance(param, datetime.datetime) and
-                                not isinstance(param, Oracle_datetime)):
+        if settings.USE_TZ and (
+            isinstance(param, datetime.datetime)
+            and not isinstance(param, Oracle_datetime)
+        ):
             param = Oracle_datetime.from_datetime(param)
 
         string_size = 0
@@ -356,7 +372,7 @@ class OracleParam:
             param = 1
         elif param is False:
             param = 0
-        if hasattr(param, 'bind_parameter'):
+        if hasattr(param, "bind_parameter"):
             self.force_bytes = param.bind_parameter(cursor)
         elif isinstance(param, (Database.Binary, datetime.timedelta)):
             self.force_bytes = param
@@ -367,7 +383,7 @@ class OracleParam:
             if isinstance(self.force_bytes, str):
                 # We could optimize by only converting up to 4000 bytes here
                 string_size = len(force_bytes(param, cursor.charset, strings_only))
-        if hasattr(param, 'input_size'):
+        if hasattr(param, "input_size"):
             # If parameter has `input_size` attribute, use that.
             self.input_size = param.input_size
         elif string_size > 4000:
@@ -397,7 +413,7 @@ class VariableWrapper:
         return getattr(self.var, key)
 
     def __setattr__(self, key, value):
-        if key == 'var':
+        if key == "var":
             self.__dict__[key] = value
         else:
             setattr(self.var, key, value)
@@ -409,7 +425,8 @@ class FormatStylePlaceholderCursor:
     style. This fixes it -- but note that if you want to use a literal "%s" in
     a query, you'll need to use "%%s".
     """
-    charset = 'utf-8'
+
+    charset = "utf-8"
 
     def __init__(self, connection):
         self.cursor = connection.cursor()
@@ -417,7 +434,7 @@ class FormatStylePlaceholderCursor:
 
     @staticmethod
     def _output_number_converter(value):
-        return decimal.Decimal(value) if '.' in value else int(value)
+        return decimal.Decimal(value) if "." in value else int(value)
 
     @staticmethod
     def _get_decimal_converter(precision, scale):
@@ -447,7 +464,9 @@ class FormatStylePlaceholderCursor:
             elif precision > 0:
                 # NUMBER(p,s) column: decimal-precision fixed point.
                 # This comes from IntegerField and DecimalField columns.
-                outconverter = FormatStylePlaceholderCursor._get_decimal_converter(precision, scale)
+                outconverter = FormatStylePlaceholderCursor._get_decimal_converter(
+                    precision, scale
+                )
             else:
                 # No type information. This normally comes from a
                 # mathematical expression in the SELECT list. Guess int
@@ -468,7 +487,7 @@ class FormatStylePlaceholderCursor:
 
     def _guess_input_sizes(self, params_list):
         # Try dict handling; if that fails, treat as sequence
-        if hasattr(params_list[0], 'keys'):
+        if hasattr(params_list[0], "keys"):
             sizes = {}
             for params in params_list:
                 for k, value in params.items():
@@ -488,7 +507,7 @@ class FormatStylePlaceholderCursor:
 
     def _param_generator(self, params):
         # Try dict handling; if that fails, treat as sequence
-        if hasattr(params, 'items'):
+        if hasattr(params, "items"):
             return {k: v.force_bytes for k, v in params.items()}
         else:
             return [p.force_bytes for p in params]
@@ -498,11 +517,11 @@ class FormatStylePlaceholderCursor:
         # it does want a trailing ';' but not a trailing '/'.  However, these
         # characters must be included in the original query in case the query
         # is being passed to SQL*Plus.
-        if query.endswith(';') or query.endswith('/'):
+        if query.endswith(";") or query.endswith("/"):
             query = query[:-1]
         if params is None:
             params = []
-        elif hasattr(params, 'keys'):
+        elif hasattr(params, "keys"):
             # Handle params as dict
             args = {k: ":%s" % k for k in params}
             query = query % args
@@ -515,15 +534,14 @@ class FormatStylePlaceholderCursor:
             # args = [':arg0', ':arg1', ':arg0', ':arg2', ':arg0']
             # params = {':arg0': 0.75, ':arg1': 2, ':arg2': 'sth'}
             params_dict = {
-                param: ':arg%d' % i
-                for i, param in enumerate(dict.fromkeys(params))
+                param: ":arg%d" % i for i, param in enumerate(dict.fromkeys(params))
             }
             args = [params_dict[param] for param in params]
             params = {value: key for key, value in params_dict.items()}
             query = query % tuple(args)
         else:
             # Handle params as sequence
-            args = [(':arg%d' % i) for i in range(len(params))]
+            args = [(":arg%d" % i) for i in range(len(params))]
             query = query % tuple(args)
         return query, self._format_params(params)
 
@@ -545,7 +563,9 @@ class FormatStylePlaceholderCursor:
         formatted = [firstparams] + [self._format_params(p) for p in params_iter]
         self._guess_input_sizes(formatted)
         with wrap_oracle_errors():
-            return self.cursor.executemany(query, [self._param_generator(p) for p in formatted])
+            return self.cursor.executemany(
+                query, [self._param_generator(p) for p in formatted]
+            )
 
     def close(self):
         try:
