@@ -1,13 +1,11 @@
 from django.db import models
 from django.db.migrations.operations.base import Operation
 from django.db.migrations.state import ModelState
+from django.db.migrations.utils import field_references, resolve_relation
 from django.db.models.options import normalize_together
 from django.utils.functional import cached_property
 
-from .fields import (
-    AddField, AlterField, FieldOperation, RemoveField, RenameField,
-)
-from .utils import field_references, get_references, resolve_relation
+from .fields import AddField, AlterField, FieldOperation, RemoveField, RenameField
 
 
 def _check_for_duplicates(arg_name, objs):
@@ -32,16 +30,15 @@ class ModelOperation(Operation):
         return name.lower() == self.name_lower
 
     def reduce(self, operation, app_label):
-        return (
-            super().reduce(operation, app_label) or
-            not operation.references_model(self.name, app_label)
+        return super().reduce(operation, app_label) or not operation.references_model(
+            self.name, app_label
         )
 
 
 class CreateModel(ModelOperation):
     """Create a model's table."""
 
-    serialization_expand_args = ['fields', 'options', 'managers']
+    serialization_expand_args = ["fields", "options", "managers"]
 
     def __init__(self, name, fields, options=None, bases=None, managers=None):
         self.fields = fields
@@ -51,40 +48,44 @@ class CreateModel(ModelOperation):
         super().__init__(name)
         # Sanity-check that there are no duplicated field names, bases, or
         # manager names
-        _check_for_duplicates('fields', (name for name, _ in self.fields))
-        _check_for_duplicates('bases', (
-            base._meta.label_lower if hasattr(base, '_meta') else
-            base.lower() if isinstance(base, str) else base
-            for base in self.bases
-        ))
-        _check_for_duplicates('managers', (name for name, _ in self.managers))
+        _check_for_duplicates("fields", (name for name, _ in self.fields))
+        _check_for_duplicates(
+            "bases",
+            (
+                base._meta.label_lower
+                if hasattr(base, "_meta")
+                else base.lower()
+                if isinstance(base, str)
+                else base
+                for base in self.bases
+            ),
+        )
+        _check_for_duplicates("managers", (name for name, _ in self.managers))
 
     def deconstruct(self):
         kwargs = {
-            'name': self.name,
-            'fields': self.fields,
+            "name": self.name,
+            "fields": self.fields,
         }
         if self.options:
-            kwargs['options'] = self.options
+            kwargs["options"] = self.options
         if self.bases and self.bases != (models.Model,):
-            kwargs['bases'] = self.bases
-        if self.managers and self.managers != [('objects', models.Manager())]:
-            kwargs['managers'] = self.managers
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+            kwargs["bases"] = self.bases
+        if self.managers and self.managers != [("objects", models.Manager())]:
+            kwargs["managers"] = self.managers
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
-        state.add_model(ModelState(
-            app_label,
-            self.name,
-            list(self.fields),
-            dict(self.options),
-            tuple(self.bases),
-            list(self.managers),
-        ))
+        state.add_model(
+            ModelState(
+                app_label,
+                self.name,
+                list(self.fields),
+                dict(self.options),
+                tuple(self.bases),
+                list(self.managers),
+            )
+        )
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = to_state.apps.get_model(app_label, self.name)
@@ -97,7 +98,10 @@ class CreateModel(ModelOperation):
             schema_editor.delete_model(model)
 
     def describe(self):
-        return "Create %smodel %s" % ("proxy " if self.options.get("proxy", False) else "", self.name)
+        return "Create %smodel %s" % (
+            "proxy " if self.options.get("proxy", False) else "",
+            self.name,
+        )
 
     @property
     def migration_name_fragment(self):
@@ -111,22 +115,32 @@ class CreateModel(ModelOperation):
         # Check we didn't inherit from the model
         reference_model_tuple = (app_label, name_lower)
         for base in self.bases:
-            if (base is not models.Model and isinstance(base, (models.base.ModelBase, str)) and
-                    resolve_relation(base, app_label) == reference_model_tuple):
+            if (
+                base is not models.Model
+                and isinstance(base, (models.base.ModelBase, str))
+                and resolve_relation(base, app_label) == reference_model_tuple
+            ):
                 return True
 
         # Check we have no FKs/M2Ms with it
         for _name, field in self.fields:
-            if field_references((app_label, self.name_lower), field, reference_model_tuple):
+            if field_references(
+                (app_label, self.name_lower), field, reference_model_tuple
+            ):
                 return True
         return False
 
     def reduce(self, operation, app_label):
-        if (isinstance(operation, DeleteModel) and
-                self.name_lower == operation.name_lower and
-                not self.options.get("proxy", False)):
+        if (
+            isinstance(operation, DeleteModel)
+            and self.name_lower == operation.name_lower
+            and not self.options.get("proxy", False)
+        ):
             return []
-        elif isinstance(operation, RenameModel) and self.name_lower == operation.old_name_lower:
+        elif (
+            isinstance(operation, RenameModel)
+            and self.name_lower == operation.old_name_lower
+        ):
             return [
                 CreateModel(
                     operation.new_name,
@@ -136,7 +150,10 @@ class CreateModel(ModelOperation):
                     managers=self.managers,
                 ),
             ]
-        elif isinstance(operation, AlterModelOptions) and self.name_lower == operation.name_lower:
+        elif (
+            isinstance(operation, AlterModelOptions)
+            and self.name_lower == operation.name_lower
+        ):
             options = {**self.options, **operation.options}
             for key in operation.ALTER_OPTION_KEYS:
                 if key not in operation.options:
@@ -150,27 +167,42 @@ class CreateModel(ModelOperation):
                     managers=self.managers,
                 ),
             ]
-        elif isinstance(operation, AlterTogetherOptionOperation) and self.name_lower == operation.name_lower:
+        elif (
+            isinstance(operation, AlterTogetherOptionOperation)
+            and self.name_lower == operation.name_lower
+        ):
             return [
                 CreateModel(
                     self.name,
                     fields=self.fields,
-                    options={**self.options, **{operation.option_name: operation.option_value}},
+                    options={
+                        **self.options,
+                        **{operation.option_name: operation.option_value},
+                    },
                     bases=self.bases,
                     managers=self.managers,
                 ),
             ]
-        elif isinstance(operation, AlterOrderWithRespectTo) and self.name_lower == operation.name_lower:
+        elif (
+            isinstance(operation, AlterOrderWithRespectTo)
+            and self.name_lower == operation.name_lower
+        ):
             return [
                 CreateModel(
                     self.name,
                     fields=self.fields,
-                    options={**self.options, 'order_with_respect_to': operation.order_with_respect_to},
+                    options={
+                        **self.options,
+                        "order_with_respect_to": operation.order_with_respect_to,
+                    },
                     bases=self.bases,
                     managers=self.managers,
                 ),
             ]
-        elif isinstance(operation, FieldOperation) and self.name_lower == operation.model_name_lower:
+        elif (
+            isinstance(operation, FieldOperation)
+            and self.name_lower == operation.model_name_lower
+        ):
             if isinstance(operation, AddField):
                 return [
                     CreateModel(
@@ -196,17 +228,25 @@ class CreateModel(ModelOperation):
                 ]
             elif isinstance(operation, RemoveField):
                 options = self.options.copy()
-                for option_name in ('unique_together', 'index_together'):
+                for option_name in ("unique_together", "index_together"):
                     option = options.pop(option_name, None)
                     if option:
-                        option = set(filter(bool, (
-                            tuple(f for f in fields if f != operation.name_lower) for fields in option
-                        )))
+                        option = set(
+                            filter(
+                                bool,
+                                (
+                                    tuple(
+                                        f for f in fields if f != operation.name_lower
+                                    )
+                                    for fields in option
+                                ),
+                            )
+                        )
                         if option:
                             options[option_name] = option
-                order_with_respect_to = options.get('order_with_respect_to')
+                order_with_respect_to = options.get("order_with_respect_to")
                 if order_with_respect_to == operation.name_lower:
-                    del options['order_with_respect_to']
+                    del options["order_with_respect_to"]
                 return [
                     CreateModel(
                         self.name,
@@ -222,16 +262,19 @@ class CreateModel(ModelOperation):
                 ]
             elif isinstance(operation, RenameField):
                 options = self.options.copy()
-                for option_name in ('unique_together', 'index_together'):
+                for option_name in ("unique_together", "index_together"):
                     option = options.get(option_name)
                     if option:
                         options[option_name] = {
-                            tuple(operation.new_name if f == operation.old_name else f for f in fields)
+                            tuple(
+                                operation.new_name if f == operation.old_name else f
+                                for f in fields
+                            )
                             for fields in option
                         }
-                order_with_respect_to = options.get('order_with_respect_to')
+                order_with_respect_to = options.get("order_with_respect_to")
                 if order_with_respect_to == operation.old_name:
-                    options['order_with_respect_to'] = operation.new_name
+                    options["order_with_respect_to"] = operation.new_name
                 return [
                     CreateModel(
                         self.name,
@@ -252,13 +295,9 @@ class DeleteModel(ModelOperation):
 
     def deconstruct(self):
         kwargs = {
-            'name': self.name,
+            "name": self.name,
         }
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
         state.remove_model(app_label, self.name_lower)
@@ -283,7 +322,7 @@ class DeleteModel(ModelOperation):
 
     @property
     def migration_name_fragment(self):
-        return 'delete_%s' % self.name_lower
+        return "delete_%s" % self.name_lower
 
 
 class RenameModel(ModelOperation):
@@ -304,41 +343,13 @@ class RenameModel(ModelOperation):
 
     def deconstruct(self):
         kwargs = {
-            'old_name': self.old_name,
-            'new_name': self.new_name,
+            "old_name": self.old_name,
+            "new_name": self.new_name,
         }
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
-        # Add a new model.
-        renamed_model = state.models[app_label, self.old_name_lower].clone()
-        renamed_model.name = self.new_name
-        state.models[app_label, self.new_name_lower] = renamed_model
-        # Repoint all fields pointing to the old model to the new one.
-        old_model_tuple = (app_label, self.old_name_lower)
-        new_remote_model = '%s.%s' % (app_label, self.new_name)
-        to_reload = set()
-        for model_state, name, field, reference in get_references(state, old_model_tuple):
-            changed_field = None
-            if reference.to:
-                changed_field = field.clone()
-                changed_field.remote_field.model = new_remote_model
-            if reference.through:
-                if changed_field is None:
-                    changed_field = field.clone()
-                changed_field.remote_field.through = new_remote_model
-            if changed_field:
-                model_state.fields[name] = changed_field
-                to_reload.add((model_state.app_label, model_state.name_lower))
-        # Reload models related to old model before removing the old model.
-        state.reload_models(to_reload, delay=True)
-        # Remove the old model.
-        state.remove_model(app_label, self.old_name_lower)
-        state.reload_model(app_label, self.new_name_lower, delay=True)
+        state.rename_model(app_label, self.old_name, self.new_name)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         new_model = to_state.apps.get_model(app_label, self.new_name)
@@ -361,19 +372,24 @@ class RenameModel(ModelOperation):
                         related_object.related_model._meta.app_label,
                         related_object.related_model._meta.model_name,
                     )
-                to_field = to_state.apps.get_model(
-                    *related_key
-                )._meta.get_field(related_object.field.name)
+                to_field = to_state.apps.get_model(*related_key)._meta.get_field(
+                    related_object.field.name
+                )
                 schema_editor.alter_field(
                     model,
                     related_object.field,
                     to_field,
                 )
             # Rename M2M fields whose name is based on this model's name.
-            fields = zip(old_model._meta.local_many_to_many, new_model._meta.local_many_to_many)
+            fields = zip(
+                old_model._meta.local_many_to_many, new_model._meta.local_many_to_many
+            )
             for (old_field, new_field) in fields:
                 # Skip self-referential fields as these are renamed above.
-                if new_field.model == new_field.related_model or not new_field.remote_field.through._meta.auto_created:
+                if (
+                    new_field.model == new_field.related_model
+                    or not new_field.remote_field.through._meta.auto_created
+                ):
                     continue
                 # Rename the M2M table that's based on this model's name.
                 old_m2m_model = old_field.remote_field.through
@@ -392,18 +408,23 @@ class RenameModel(ModelOperation):
                 )
 
     def database_backwards(self, app_label, schema_editor, from_state, to_state):
-        self.new_name_lower, self.old_name_lower = self.old_name_lower, self.new_name_lower
+        self.new_name_lower, self.old_name_lower = (
+            self.old_name_lower,
+            self.new_name_lower,
+        )
         self.new_name, self.old_name = self.old_name, self.new_name
 
         self.database_forwards(app_label, schema_editor, from_state, to_state)
 
-        self.new_name_lower, self.old_name_lower = self.old_name_lower, self.new_name_lower
+        self.new_name_lower, self.old_name_lower = (
+            self.old_name_lower,
+            self.new_name_lower,
+        )
         self.new_name, self.old_name = self.old_name, self.new_name
 
     def references_model(self, name, app_label):
         return (
-            name.lower() == self.old_name_lower or
-            name.lower() == self.new_name_lower
+            name.lower() == self.old_name_lower or name.lower() == self.new_name_lower
         )
 
     def describe(self):
@@ -411,11 +432,13 @@ class RenameModel(ModelOperation):
 
     @property
     def migration_name_fragment(self):
-        return 'rename_%s_%s' % (self.old_name_lower, self.new_name_lower)
+        return "rename_%s_%s" % (self.old_name_lower, self.new_name_lower)
 
     def reduce(self, operation, app_label):
-        if (isinstance(operation, RenameModel) and
-                self.new_name_lower == operation.old_name_lower):
+        if (
+            isinstance(operation, RenameModel)
+            and self.new_name_lower == operation.old_name_lower
+        ):
             return [
                 RenameModel(
                     self.old_name,
@@ -424,15 +447,17 @@ class RenameModel(ModelOperation):
             ]
         # Skip `ModelOperation.reduce` as we want to run `references_model`
         # against self.new_name.
-        return (
-            super(ModelOperation, self).reduce(operation, app_label) or
-            not operation.references_model(self.new_name, app_label)
-        )
+        return super(ModelOperation, self).reduce(
+            operation, app_label
+        ) or not operation.references_model(self.new_name, app_label)
 
 
 class ModelOptionOperation(ModelOperation):
     def reduce(self, operation, app_label):
-        if isinstance(operation, (self.__class__, DeleteModel)) and self.name_lower == operation.name_lower:
+        if (
+            isinstance(operation, (self.__class__, DeleteModel))
+            and self.name_lower == operation.name_lower
+        ):
             return [operation]
         return super().reduce(operation, app_label)
 
@@ -446,18 +471,13 @@ class AlterModelTable(ModelOptionOperation):
 
     def deconstruct(self):
         kwargs = {
-            'name': self.name,
-            'table': self.table,
+            "name": self.name,
+            "table": self.table,
         }
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
-        state.models[app_label, self.name_lower].options["db_table"] = self.table
-        state.reload_model(app_label, self.name_lower, delay=True)
+        state.alter_model_options(app_label, self.name_lower, {"db_table": self.table})
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         new_model = to_state.apps.get_model(app_label, self.name)
@@ -469,7 +489,9 @@ class AlterModelTable(ModelOptionOperation):
                 new_model._meta.db_table,
             )
             # Rename M2M fields whose name is based on this model's db_table
-            for (old_field, new_field) in zip(old_model._meta.local_many_to_many, new_model._meta.local_many_to_many):
+            for (old_field, new_field) in zip(
+                old_model._meta.local_many_to_many, new_model._meta.local_many_to_many
+            ):
                 if new_field.remote_field.through._meta.auto_created:
                     schema_editor.alter_db_table(
                         new_field.remote_field.through,
@@ -483,12 +505,12 @@ class AlterModelTable(ModelOptionOperation):
     def describe(self):
         return "Rename table for %s to %s" % (
             self.name,
-            self.table if self.table is not None else "(default)"
+            self.table if self.table is not None else "(default)",
         )
 
     @property
     def migration_name_fragment(self):
-        return 'alter_%s_table' % self.name_lower
+        return "alter_%s_table" % self.name_lower
 
 
 class AlterTogetherOptionOperation(ModelOptionOperation):
@@ -506,25 +528,23 @@ class AlterTogetherOptionOperation(ModelOptionOperation):
 
     def deconstruct(self):
         kwargs = {
-            'name': self.name,
+            "name": self.name,
             self.option_name: self.option_value,
         }
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.name_lower]
-        model_state.options[self.option_name] = self.option_value
-        state.reload_model(app_label, self.name_lower, delay=True)
+        state.alter_model_options(
+            app_label,
+            self.name_lower,
+            {self.option_name: self.option_value},
+        )
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         new_model = to_state.apps.get_model(app_label, self.name)
         if self.allow_migrate_model(schema_editor.connection.alias, new_model):
             old_model = from_state.apps.get_model(app_label, self.name)
-            alter_together = getattr(schema_editor, 'alter_%s' % self.option_name)
+            alter_together = getattr(schema_editor, "alter_%s" % self.option_name)
             alter_together(
                 new_model,
                 getattr(old_model._meta, self.option_name, set()),
@@ -535,20 +555,21 @@ class AlterTogetherOptionOperation(ModelOptionOperation):
         return self.database_forwards(app_label, schema_editor, from_state, to_state)
 
     def references_field(self, model_name, name, app_label):
-        return (
-            self.references_model(model_name, app_label) and
-            (
-                not self.option_value or
-                any((name in fields) for fields in self.option_value)
-            )
+        return self.references_model(model_name, app_label) and (
+            not self.option_value
+            or any((name in fields) for fields in self.option_value)
         )
 
     def describe(self):
-        return "Alter %s for %s (%s constraint(s))" % (self.option_name, self.name, len(self.option_value or ''))
+        return "Alter %s for %s (%s constraint(s))" % (
+            self.option_name,
+            self.name,
+            len(self.option_value or ""),
+        )
 
     @property
     def migration_name_fragment(self):
-        return 'alter_%s_%s' % (self.name_lower, self.option_name)
+        return "alter_%s_%s" % (self.name_lower, self.option_name)
 
 
 class AlterUniqueTogether(AlterTogetherOptionOperation):
@@ -556,7 +577,8 @@ class AlterUniqueTogether(AlterTogetherOptionOperation):
     Change the value of unique_together to the target one.
     Input value of unique_together must be a set of tuples.
     """
-    option_name = 'unique_together'
+
+    option_name = "unique_together"
 
     def __init__(self, name, unique_together):
         super().__init__(name, unique_together)
@@ -567,6 +589,7 @@ class AlterIndexTogether(AlterTogetherOptionOperation):
     Change the value of index_together to the target one.
     Input value of index_together must be a set of tuples.
     """
+
     option_name = "index_together"
 
     def __init__(self, name, index_together):
@@ -576,7 +599,7 @@ class AlterIndexTogether(AlterTogetherOptionOperation):
 class AlterOrderWithRespectTo(ModelOptionOperation):
     """Represent a change with the order_with_respect_to option."""
 
-    option_name = 'order_with_respect_to'
+    option_name = "order_with_respect_to"
 
     def __init__(self, name, order_with_respect_to):
         self.order_with_respect_to = order_with_respect_to
@@ -584,30 +607,36 @@ class AlterOrderWithRespectTo(ModelOptionOperation):
 
     def deconstruct(self):
         kwargs = {
-            'name': self.name,
-            'order_with_respect_to': self.order_with_respect_to,
+            "name": self.name,
+            "order_with_respect_to": self.order_with_respect_to,
         }
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.name_lower]
-        model_state.options['order_with_respect_to'] = self.order_with_respect_to
-        state.reload_model(app_label, self.name_lower, delay=True)
+        state.alter_model_options(
+            app_label,
+            self.name_lower,
+            {self.option_name: self.order_with_respect_to},
+        )
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         to_model = to_state.apps.get_model(app_label, self.name)
         if self.allow_migrate_model(schema_editor.connection.alias, to_model):
             from_model = from_state.apps.get_model(app_label, self.name)
             # Remove a field if we need to
-            if from_model._meta.order_with_respect_to and not to_model._meta.order_with_respect_to:
-                schema_editor.remove_field(from_model, from_model._meta.get_field("_order"))
+            if (
+                from_model._meta.order_with_respect_to
+                and not to_model._meta.order_with_respect_to
+            ):
+                schema_editor.remove_field(
+                    from_model, from_model._meta.get_field("_order")
+                )
             # Add a field if we need to (altering the column is untouched as
             # it's likely a rename)
-            elif to_model._meta.order_with_respect_to and not from_model._meta.order_with_respect_to:
+            elif (
+                to_model._meta.order_with_respect_to
+                and not from_model._meta.order_with_respect_to
+            ):
                 field = to_model._meta.get_field("_order")
                 if not field.has_default():
                     field.default = 0
@@ -620,20 +649,19 @@ class AlterOrderWithRespectTo(ModelOptionOperation):
         self.database_forwards(app_label, schema_editor, from_state, to_state)
 
     def references_field(self, model_name, name, app_label):
-        return (
-            self.references_model(model_name, app_label) and
-            (
-                self.order_with_respect_to is None or
-                name == self.order_with_respect_to
-            )
+        return self.references_model(model_name, app_label) and (
+            self.order_with_respect_to is None or name == self.order_with_respect_to
         )
 
     def describe(self):
-        return "Set order_with_respect_to on %s to %s" % (self.name, self.order_with_respect_to)
+        return "Set order_with_respect_to on %s to %s" % (
+            self.name,
+            self.order_with_respect_to,
+        )
 
     @property
     def migration_name_fragment(self):
-        return 'alter_%s_order_with_respect_to' % self.name_lower
+        return "alter_%s_order_with_respect_to" % self.name_lower
 
 
 class AlterModelOptions(ModelOptionOperation):
@@ -664,22 +692,18 @@ class AlterModelOptions(ModelOptionOperation):
 
     def deconstruct(self):
         kwargs = {
-            'name': self.name,
-            'options': self.options,
+            "name": self.name,
+            "options": self.options,
         }
-        return (
-            self.__class__.__qualname__,
-            [],
-            kwargs
-        )
+        return (self.__class__.__qualname__, [], kwargs)
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.name_lower]
-        model_state.options = {**model_state.options, **self.options}
-        for key in self.ALTER_OPTION_KEYS:
-            if key not in self.options:
-                model_state.options.pop(key, False)
-        state.reload_model(app_label, self.name_lower, delay=True)
+        state.alter_model_options(
+            app_label,
+            self.name_lower,
+            self.options,
+            self.ALTER_OPTION_KEYS,
+        )
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         pass
@@ -692,29 +716,23 @@ class AlterModelOptions(ModelOptionOperation):
 
     @property
     def migration_name_fragment(self):
-        return 'alter_%s_options' % self.name_lower
+        return "alter_%s_options" % self.name_lower
 
 
 class AlterModelManagers(ModelOptionOperation):
     """Alter the model's managers."""
 
-    serialization_expand_args = ['managers']
+    serialization_expand_args = ["managers"]
 
     def __init__(self, name, managers):
         self.managers = managers
         super().__init__(name)
 
     def deconstruct(self):
-        return (
-            self.__class__.__qualname__,
-            [self.name, self.managers],
-            {}
-        )
+        return (self.__class__.__qualname__, [self.name, self.managers], {})
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.name_lower]
-        model_state.managers = list(self.managers)
-        state.reload_model(app_label, self.name_lower, delay=True)
+        state.alter_model_managers(app_label, self.name_lower, self.managers)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         pass
@@ -727,11 +745,11 @@ class AlterModelManagers(ModelOptionOperation):
 
     @property
     def migration_name_fragment(self):
-        return 'alter_%s_managers' % self.name_lower
+        return "alter_%s_managers" % self.name_lower
 
 
 class IndexOperation(Operation):
-    option_name = 'indexes'
+    option_name = "indexes"
 
     @cached_property
     def model_name_lower(self):
@@ -751,9 +769,7 @@ class AddIndex(IndexOperation):
         self.index = index
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.model_name_lower]
-        model_state.options[self.option_name] = [*model_state.options[self.option_name], self.index.clone()]
-        state.reload_model(app_label, self.model_name_lower, delay=True)
+        state.add_index(app_label, self.model_name_lower, self.index)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = to_state.apps.get_model(app_label, self.model_name)
@@ -767,8 +783,8 @@ class AddIndex(IndexOperation):
 
     def deconstruct(self):
         kwargs = {
-            'model_name': self.model_name,
-            'index': self.index,
+            "model_name": self.model_name,
+            "index": self.index,
         }
         return (
             self.__class__.__qualname__,
@@ -778,20 +794,20 @@ class AddIndex(IndexOperation):
 
     def describe(self):
         if self.index.expressions:
-            return 'Create index %s on %s on model %s' % (
+            return "Create index %s on %s on model %s" % (
                 self.index.name,
-                ', '.join([str(expression) for expression in self.index.expressions]),
+                ", ".join([str(expression) for expression in self.index.expressions]),
                 self.model_name,
             )
-        return 'Create index %s on field(s) %s of model %s' % (
+        return "Create index %s on field(s) %s of model %s" % (
             self.index.name,
-            ', '.join(self.index.fields),
+            ", ".join(self.index.fields),
             self.model_name,
         )
 
     @property
     def migration_name_fragment(self):
-        return '%s_%s' % (self.model_name_lower, self.index.name.lower())
+        return "%s_%s" % (self.model_name_lower, self.index.name.lower())
 
 
 class RemoveIndex(IndexOperation):
@@ -802,10 +818,7 @@ class RemoveIndex(IndexOperation):
         self.name = name
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.model_name_lower]
-        indexes = model_state.options[self.option_name]
-        model_state.options[self.option_name] = [idx for idx in indexes if idx.name != self.name]
-        state.reload_model(app_label, self.model_name_lower, delay=True)
+        state.remove_index(app_label, self.model_name_lower, self.name)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = from_state.apps.get_model(app_label, self.model_name)
@@ -823,8 +836,8 @@ class RemoveIndex(IndexOperation):
 
     def deconstruct(self):
         kwargs = {
-            'model_name': self.model_name,
-            'name': self.name,
+            "model_name": self.model_name,
+            "name": self.name,
         }
         return (
             self.__class__.__qualname__,
@@ -833,24 +846,22 @@ class RemoveIndex(IndexOperation):
         )
 
     def describe(self):
-        return 'Remove index %s from %s' % (self.name, self.model_name)
+        return "Remove index %s from %s" % (self.name, self.model_name)
 
     @property
     def migration_name_fragment(self):
-        return 'remove_%s_%s' % (self.model_name_lower, self.name.lower())
+        return "remove_%s_%s" % (self.model_name_lower, self.name.lower())
 
 
 class AddConstraint(IndexOperation):
-    option_name = 'constraints'
+    option_name = "constraints"
 
     def __init__(self, model_name, constraint):
         self.model_name = model_name
         self.constraint = constraint
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.model_name_lower]
-        model_state.options[self.option_name] = [*model_state.options[self.option_name], self.constraint]
-        state.reload_model(app_label, self.model_name_lower, delay=True)
+        state.add_constraint(app_label, self.model_name_lower, self.constraint)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = to_state.apps.get_model(app_label, self.model_name)
@@ -863,31 +874,35 @@ class AddConstraint(IndexOperation):
             schema_editor.remove_constraint(model, self.constraint)
 
     def deconstruct(self):
-        return self.__class__.__name__, [], {
-            'model_name': self.model_name,
-            'constraint': self.constraint,
-        }
+        return (
+            self.__class__.__name__,
+            [],
+            {
+                "model_name": self.model_name,
+                "constraint": self.constraint,
+            },
+        )
 
     def describe(self):
-        return 'Create constraint %s on model %s' % (self.constraint.name, self.model_name)
+        return "Create constraint %s on model %s" % (
+            self.constraint.name,
+            self.model_name,
+        )
 
     @property
     def migration_name_fragment(self):
-        return '%s_%s' % (self.model_name_lower, self.constraint.name.lower())
+        return "%s_%s" % (self.model_name_lower, self.constraint.name.lower())
 
 
 class RemoveConstraint(IndexOperation):
-    option_name = 'constraints'
+    option_name = "constraints"
 
     def __init__(self, model_name, name):
         self.model_name = model_name
         self.name = name
 
     def state_forwards(self, app_label, state):
-        model_state = state.models[app_label, self.model_name_lower]
-        constraints = model_state.options[self.option_name]
-        model_state.options[self.option_name] = [c for c in constraints if c.name != self.name]
-        state.reload_model(app_label, self.model_name_lower, delay=True)
+        state.remove_constraint(app_label, self.model_name_lower, self.name)
 
     def database_forwards(self, app_label, schema_editor, from_state, to_state):
         model = to_state.apps.get_model(app_label, self.model_name)
@@ -904,14 +919,18 @@ class RemoveConstraint(IndexOperation):
             schema_editor.add_constraint(model, constraint)
 
     def deconstruct(self):
-        return self.__class__.__name__, [], {
-            'model_name': self.model_name,
-            'name': self.name,
-        }
+        return (
+            self.__class__.__name__,
+            [],
+            {
+                "model_name": self.model_name,
+                "name": self.name,
+            },
+        )
 
     def describe(self):
-        return 'Remove constraint %s from model %s' % (self.name, self.model_name)
+        return "Remove constraint %s from model %s" % (self.name, self.model_name)
 
     @property
     def migration_name_fragment(self):
-        return 'remove_%s_%s' % (self.model_name_lower, self.name.lower())
+        return "remove_%s_%s" % (self.model_name_lower, self.name.lower())
