@@ -1,11 +1,16 @@
 import copy
+from collections import defaultdict
 from contextlib import contextmanager
+from functools import partial
 
 from django.apps import AppConfig
-from django.apps.registry import Apps, apps as global_apps
+from django.apps.registry import Apps
+from django.apps.registry import apps as global_apps
 from django.conf import settings
+from django.core.exceptions import FieldDoesNotExist
 from django.db import models
-from django.db.models.fields.proxy import OrderWrt
+from django.db.migrations.utils import field_is_referenced, get_references
+from django.db.models import NOT_PROVIDED
 from django.db.models.fields.related import RECURSIVE_RELATIONSHIP_CONSTANT
 from django.db.models.options import DEFAULT_NAMES, normalize_together
 from django.db.models.utils import make_model_tuple
@@ -14,11 +19,12 @@ from django.utils.module_loading import import_string
 from django.utils.version import get_docs_version
 
 from .exceptions import InvalidBasesError
+from .utils import resolve_relation
 
 
-def _get_app_label_and_model_name(model, app_label=''):
+def _get_app_label_and_model_name(model, app_label=""):
     if isinstance(model, str):
-        split = model.split('.', 1)
+        split = model.split(".", 1)
         return tuple(split) if len(split) == 2 else (app_label, split[0])
     else:
         return model._meta.app_label, model._meta.model_name
@@ -27,12 +33,17 @@ def _get_app_label_and_model_name(model, app_label=''):
 def _get_related_models(m):
     """Return all models that have a direct relationship to the given model."""
     related_models = [
-        subclass for subclass in m.__subclasses__()
+        subclass
+        for subclass in m.__subclasses__()
         if issubclass(subclass, models.Model)
     ]
     related_fields_models = set()
     for f in m._meta.get_fields(include_parents=True, include_hidden=True):
-        if f.is_relation and f.related_model is not None and not isinstance(f.related_model, str):
+        if (
+            f.is_relation
+            and f.related_model is not None
+            and not isinstance(f.related_model, str)
+        ):
             related_fields_models.add(f.model)
             related_models.append(f.related_model)
     # Reverse accessors of foreign keys to proxy models are attached to their
@@ -68,7 +79,10 @@ def get_related_models_recursive(model):
     seen = set()
     queue = _get_related_models(model)
     for rel_mod in queue:
-        rel_app_label, rel_model_name = rel_mod._meta.app_label, rel_mod._meta.model_name
+        rel_app_label, rel_model_name = (
+            rel_mod._meta.app_label,
+            rel_mod._meta.model_name,
+        )
         if (rel_app_label, rel_model_name) in seen:
             continue
         seen.add((rel_app_label, rel_model_name))
@@ -86,23 +100,228 @@ class ProjectState:
     def __init__(self, models=None, real_apps=None):
         self.models = models or {}
         # Apps to include from main registry, usually unmigrated ones
-        self.real_apps = real_apps or []
+        if real_apps is None:
+            real_apps = set()
+        else:
+            assert isinstance(real_apps, set)
+        self.real_apps = real_apps
         self.is_delayed = False
+        # {remote_model_key: {model_key: {field_name: field}}}
+        self._relations = None
+
+    @property
+    def relations(self):
+        if self._relations is None:
+            self.resolve_fields_and_relations()
+        return self._relations
 
     def add_model(self, model_state):
-        app_label, model_name = model_state.app_label, model_state.name_lower
-        self.models[(app_label, model_name)] = model_state
-        if 'apps' in self.__dict__:  # hasattr would cache the property
-            self.reload_model(app_label, model_name)
+        model_key = model_state.app_label, model_state.name_lower
+        self.models[model_key] = model_state
+        if self._relations is not None:
+            self.resolve_model_relations(model_key)
+        if "apps" in self.__dict__:  # hasattr would cache the property
+            self.reload_model(*model_key)
 
     def remove_model(self, app_label, model_name):
-        del self.models[app_label, model_name]
-        if 'apps' in self.__dict__:  # hasattr would cache the property
-            self.apps.unregister_model(app_label, model_name)
+        model_key = app_label, model_name
+        del self.models[model_key]
+        if self._relations is not None:
+            self._relations.pop(model_key, None)
+            # Call list() since _relations can change size during iteration.
+            for related_model_key, model_relations in list(self._relations.items()):
+                model_relations.pop(model_key, None)
+                if not model_relations:
+                    del self._relations[related_model_key]
+        if "apps" in self.__dict__:  # hasattr would cache the property
+            self.apps.unregister_model(*model_key)
             # Need to do this explicitly since unregister_model() doesn't clear
             # the cache automatically (#24513)
             self.apps.clear_cache()
 
+    def rename_model(self, app_label, old_name, new_name):
+        # Add a new model.
+        old_name_lower = old_name.lower()
+        new_name_lower = new_name.lower()
+        renamed_model = self.models[app_label, old_name_lower].clone()
+        renamed_model.name = new_name
+        self.models[app_label, new_name_lower] = renamed_model
+        # Repoint all fields pointing to the old model to the new one.
+        old_model_tuple = (app_label, old_name_lower)
+        new_remote_model = f"{app_label}.{new_name}"
+        to_reload = set()
+        for model_state, name, field, reference in get_references(
+            self, old_model_tuple
+        ):
+            changed_field = None
+            if reference.to:
+                changed_field = field.clone()
+                changed_field.remote_field.model = new_remote_model
+            if reference.through:
+                if changed_field is None:
+                    changed_field = field.clone()
+                changed_field.remote_field.through = new_remote_model
+            if changed_field:
+                model_state.fields[name] = changed_field
+                to_reload.add((model_state.app_label, model_state.name_lower))
+        if self._relations is not None:
+            old_name_key = app_label, old_name_lower
+            new_name_key = app_label, new_name_lower
+            if old_name_key in self._relations:
+                self._relations[new_name_key] = self._relations.pop(old_name_key)
+            for model_relations in self._relations.values():
+                if old_name_key in model_relations:
+                    model_relations[new_name_key] = model_relations.pop(old_name_key)
+        # Reload models related to old model before removing the old model.
+        self.reload_models(to_reload, delay=True)
+        # Remove the old model.
+        self.remove_model(app_label, old_name_lower)
+        self.reload_model(app_label, new_name_lower, delay=True)
+
+    def alter_model_options(self, app_label, model_name, options, option_keys=None):
+        model_state = self.models[app_label, model_name]
+        model_state.options = {**model_state.options, **options}
+        if option_keys:
+            for key in option_keys:
+                if key not in options:
+                    model_state.options.pop(key, False)
+        self.reload_model(app_label, model_name, delay=True)
+
+    def alter_model_managers(self, app_label, model_name, managers):
+        model_state = self.models[app_label, model_name]
+        model_state.managers = list(managers)
+        self.reload_model(app_label, model_name, delay=True)
+
+    def _append_option(self, app_label, model_name, option_name, obj):
+        model_state = self.models[app_label, model_name]
+        model_state.options[option_name] = [*model_state.options[option_name], obj]
+        self.reload_model(app_label, model_name, delay=True)
+
+    def _remove_option(self, app_label, model_name, option_name, obj_name):
+        model_state = self.models[app_label, model_name]
+        objs = model_state.options[option_name]
+        model_state.options[option_name] = [obj for obj in objs if obj.name != obj_name]
+        self.reload_model(app_label, model_name, delay=True)
+
+    def add_index(self, app_label, model_name, index):
+        self._append_option(app_label, model_name, "indexes", index)
+
+    def remove_index(self, app_label, model_name, index_name):
+        self._remove_option(app_label, model_name, "indexes", index_name)
+
+    def add_constraint(self, app_label, model_name, constraint):
+        self._append_option(app_label, model_name, "constraints", constraint)
+
+    def remove_constraint(self, app_label, model_name, constraint_name):
+        self._remove_option(app_label, model_name, "constraints", constraint_name)
+
+    def add_field(self, app_label, model_name, name, field, preserve_default):
+        # If preserve default is off, don't use the default for future state.
+        if not preserve_default:
+            field = field.clone()
+            field.default = NOT_PROVIDED
+        else:
+            field = field
+        model_key = app_label, model_name
+        self.models[model_key].fields[name] = field
+        if self._relations is not None:
+            self.resolve_model_field_relations(model_key, name, field)
+        # Delay rendering of relationships if it's not a relational field.
+        delay = not field.is_relation
+        self.reload_model(*model_key, delay=delay)
+
+    def remove_field(self, app_label, model_name, name):
+        model_key = app_label, model_name
+        model_state = self.models[model_key]
+        old_field = model_state.fields.pop(name)
+        if self._relations is not None:
+            self.resolve_model_field_relations(model_key, name, old_field)
+        # Delay rendering of relationships if it's not a relational field.
+        delay = not old_field.is_relation
+        self.reload_model(*model_key, delay=delay)
+
+    def alter_field(self, app_label, model_name, name, field, preserve_default):
+        if not preserve_default:
+            field = field.clone()
+            field.default = NOT_PROVIDED
+        else:
+            field = field
+        model_key = app_label, model_name
+        fields = self.models[model_key].fields
+        if self._relations is not None:
+            old_field = fields.pop(name)
+            if old_field.is_relation:
+                self.resolve_model_field_relations(model_key, name, old_field)
+            fields[name] = field
+            if field.is_relation:
+                self.resolve_model_field_relations(model_key, name, field)
+        else:
+            fields[name] = field
+        # TODO: investigate if old relational fields must be reloaded or if
+        # it's sufficient if the new field is (#27737).
+        # Delay rendering of relationships if it's not a relational field and
+        # not referenced by a foreign key.
+        delay = not field.is_relation and not field_is_referenced(
+            self, model_key, (name, field)
+        )
+        self.reload_model(*model_key, delay=delay)
+
+    def rename_field(self, app_label, model_name, old_name, new_name):
+        model_key = app_label, model_name
+        model_state = self.models[model_key]
+        # Rename the field.
+        fields = model_state.fields
+        try:
+            found = fields.pop(old_name)
+        except KeyError:
+            raise FieldDoesNotExist(
+                f"{app_label}.{model_name} has no field named '{old_name}'"
+            )
+        fields[new_name] = found
+        for field in fields.values():
+            # Fix from_fields to refer to the new field.
+            from_fields = getattr(field, "from_fields", None)
+            if from_fields:
+                field.from_fields = tuple(
+                    [
+                        new_name if from_field_name == old_name else from_field_name
+                        for from_field_name in from_fields
+                    ]
+                )
+        # Fix index/unique_together to refer to the new field.
+        options = model_state.options
+        for option in ("index_together", "unique_together"):
+            if option in options:
+                options[option] = [
+                    [new_name if n == old_name else n for n in together]
+                    for together in options[option]
+                ]
+        # Fix to_fields to refer to the new field.
+        delay = True
+        references = get_references(self, model_key, (old_name, found))
+        for *_, field, reference in references:
+            delay = False
+            if reference.to:
+                remote_field, to_fields = reference.to
+                if getattr(remote_field, "field_name", None) == old_name:
+                    remote_field.field_name = new_name
+                if to_fields:
+                    field.to_fields = tuple(
+                        [
+                            new_name if to_field_name == old_name else to_field_name
+                            for to_field_name in to_fields
+                        ]
+                    )
+        if self._relations is not None:
+            old_name_lower = old_name.lower()
+            new_name_lower = new_name.lower()
+            for to_model in self._relations.values():
+                if old_name_lower in to_model[model_key]:
+                    field = to_model[model_key].pop(old_name_lower)
+                    field.name = new_name_lower
+                    to_model[model_key][new_name_lower] = field
+        self.reload_model(*model_key, delay=delay)
+
     def _find_reload_model(self, app_label, model_name, delay=False):
         if delay:
             self.is_delayed = True
@@ -126,11 +345,13 @@ class ProjectState:
         # Directly related models are the models pointed to by ForeignKeys,
         # OneToOneFields, and ManyToManyFields.
         direct_related_models = set()
-        for name, field in model_state.fields:
+        for field in model_state.fields.values():
             if field.is_relation:
                 if field.remote_field.model == RECURSIVE_RELATIONSHIP_CONSTANT:
                     continue
-                rel_app_label, rel_model_name = _get_app_label_and_model_name(field.related_model, app_label)
+                rel_app_label, rel_model_name = _get_app_label_and_model_name(
+                    field.related_model, app_label
+                )
                 direct_related_models.add((rel_app_label, rel_model_name.lower()))
 
         # For all direct related models recursively get all related models.
@@ -152,15 +373,17 @@ class ProjectState:
         return related_models
 
     def reload_model(self, app_label, model_name, delay=False):
-        if 'apps' in self.__dict__:  # hasattr would cache the property
+        if "apps" in self.__dict__:  # hasattr would cache the property
             related_models = self._find_reload_model(app_label, model_name, delay)
             self._reload(related_models)
 
     def reload_models(self, models, delay=True):
-        if 'apps' in self.__dict__:  # hasattr would cache the property
+        if "apps" in self.__dict__:  # hasattr would cache the property
             related_models = set()
             for app_label, model_name in models:
-                related_models.update(self._find_reload_model(app_label, model_name, delay))
+                related_models.update(
+                    self._find_reload_model(app_label, model_name, delay)
+                )
             self._reload(related_models)
 
     def _reload(self, related_models):
@@ -189,30 +412,137 @@ class ProjectState:
         # Render all models
         self.apps.render_multiple(states_to_be_rendered)
 
+    def update_model_field_relation(
+        self,
+        model,
+        model_key,
+        field_name,
+        field,
+        concretes,
+    ):
+        remote_model_key = resolve_relation(model, *model_key)
+        if remote_model_key[0] not in self.real_apps and remote_model_key in concretes:
+            remote_model_key = concretes[remote_model_key]
+        relations_to_remote_model = self._relations[remote_model_key]
+        if field_name in self.models[model_key].fields:
+            # The assert holds because it's a new relation, or an altered
+            # relation, in which case references have been removed by
+            # alter_field().
+            assert field_name not in relations_to_remote_model[model_key]
+            relations_to_remote_model[model_key][field_name] = field
+        else:
+            del relations_to_remote_model[model_key][field_name]
+            if not relations_to_remote_model[model_key]:
+                del relations_to_remote_model[model_key]
+
+    def resolve_model_field_relations(
+        self,
+        model_key,
+        field_name,
+        field,
+        concretes=None,
+    ):
+        remote_field = field.remote_field
+        if not remote_field:
+            return
+        if concretes is None:
+            concretes, _ = self._get_concrete_models_mapping_and_proxy_models()
+
+        self.update_model_field_relation(
+            remote_field.model,
+            model_key,
+            field_name,
+            field,
+            concretes,
+        )
+
+        through = getattr(remote_field, "through", None)
+        if not through:
+            return
+        self.update_model_field_relation(
+            through, model_key, field_name, field, concretes
+        )
+
+    def resolve_model_relations(self, model_key, concretes=None):
+        if concretes is None:
+            concretes, _ = self._get_concrete_models_mapping_and_proxy_models()
+
+        model_state = self.models[model_key]
+        for field_name, field in model_state.fields.items():
+            self.resolve_model_field_relations(model_key, field_name, field, concretes)
+
+    def resolve_fields_and_relations(self):
+        # Resolve fields.
+        for model_state in self.models.values():
+            for field_name, field in model_state.fields.items():
+                field.name = field_name
+        # Resolve relations.
+        # {remote_model_key: {model_key: {field_name: field}}}
+        self._relations = defaultdict(partial(defaultdict, dict))
+        concretes, proxies = self._get_concrete_models_mapping_and_proxy_models()
+
+        for model_key in concretes:
+            self.resolve_model_relations(model_key, concretes)
+
+        for model_key in proxies:
+            self._relations[model_key] = self._relations[concretes[model_key]]
+
+    def get_concrete_model_key(self, model):
+        (
+            concrete_models_mapping,
+            _,
+        ) = self._get_concrete_models_mapping_and_proxy_models()
+        model_key = make_model_tuple(model)
+        return concrete_models_mapping[model_key]
+
+    def _get_concrete_models_mapping_and_proxy_models(self):
+        concrete_models_mapping = {}
+        proxy_models = {}
+        # Split models to proxy and concrete models.
+        for model_key, model_state in self.models.items():
+            if model_state.options.get("proxy"):
+                proxy_models[model_key] = model_state
+                # Find a concrete model for the proxy.
+                concrete_models_mapping[
+                    model_key
+                ] = self._find_concrete_model_from_proxy(
+                    proxy_models,
+                    model_state,
+                )
+            else:
+                concrete_models_mapping[model_key] = model_key
+        return concrete_models_mapping, proxy_models
+
+    def _find_concrete_model_from_proxy(self, proxy_models, model_state):
+        for base in model_state.bases:
+            if not (isinstance(base, str) or issubclass(base, models.Model)):
+                continue
+            base_key = make_model_tuple(base)
+            base_state = proxy_models.get(base_key)
+            if not base_state:
+                # Concrete model found, stop looking at bases.
+                return base_key
+            return self._find_concrete_model_from_proxy(proxy_models, base_state)
+
     def clone(self):
         """Return an exact copy of this ProjectState."""
         new_state = ProjectState(
             models={k: v.clone() for k, v in self.models.items()},
             real_apps=self.real_apps,
         )
-        if 'apps' in self.__dict__:
+        if "apps" in self.__dict__:
             new_state.apps = self.apps.clone()
         new_state.is_delayed = self.is_delayed
         return new_state
 
     def clear_delayed_apps_cache(self):
-        if self.is_delayed and 'apps' in self.__dict__:
-            del self.__dict__['apps']
+        if self.is_delayed and "apps" in self.__dict__:
+            del self.__dict__["apps"]
 
     @cached_property
     def apps(self):
         return StateApps(self.real_apps, self.models)
 
-    @property
-    def concrete_apps(self):
-        self.apps = StateApps(self.real_apps, self.models, ignore_swappable=True)
-        return self.apps
-
     @classmethod
     def from_apps(cls, apps):
         """Take an Apps and return a ProjectState matching it."""
@@ -223,20 +553,20 @@ class ProjectState:
         return cls(app_models)
 
     def __eq__(self, other):
-        return self.models == other.models and set(self.real_apps) == set(other.real_apps)
+        return self.models == other.models and self.real_apps == other.real_apps
 
 
 class AppConfigStub(AppConfig):
     """Stub of an AppConfig. Only provides a label and a dict of models."""
-    # Not used, but required by AppConfig.__init__
-    path = ''
 
     def __init__(self, label):
-        self.label = label
+        self.apps = None
+        self.models = {}
         # App-label and app-name are not the same thing, so technically passing
         # in the label here is wrong. In practice, migrations don't care about
         # the app name, but we need something unique, and the label works fine.
-        super().__init__(label, None)
+        self.label = label
+        self.name = label
 
     def import_models(self):
         self.models = self.apps.all_models[self.label]
@@ -247,6 +577,7 @@ class StateApps(Apps):
     Subclass of the global Apps registry class to better handle dynamic model
     additions and removals.
     """
+
     def __init__(self, real_apps, models, ignore_swappable=False):
         # Any apps in self.real_apps should have all their models included
         # in the render. We don't use the original model instances as there
@@ -260,7 +591,9 @@ class StateApps(Apps):
                 self.real_models.append(ModelState.from_model(model, exclude_rels=True))
         # Populate the app registry with a stub for each application.
         app_labels = {model_state.app_label for model_state in models.values()}
-        app_configs = [AppConfigStub(label) for label in sorted([*real_apps, *app_labels])]
+        app_configs = [
+            AppConfigStub(label) for label in sorted([*real_apps, *app_labels])
+        ]
         super().__init__(app_configs)
 
         # These locks get in the way of copying as implemented in clone(),
@@ -273,7 +606,10 @@ class StateApps(Apps):
 
         # There shouldn't be any operations pending at this point.
         from django.core.checks.model_checks import _check_lazy_references
-        ignore = {make_model_tuple(settings.AUTH_USER_MODEL)} if ignore_swappable else set()
+
+        ignore = (
+            {make_model_tuple(settings.AUTH_USER_MODEL)} if ignore_swappable else set()
+        )
         errors = _check_lazy_references(self, ignore=ignore)
         if errors:
             raise ValueError("\n".join(error.msg for error in errors))
@@ -309,10 +645,12 @@ class StateApps(Apps):
                         new_unrendered_models.append(model)
                 if len(new_unrendered_models) == len(unrendered_models):
                     raise InvalidBasesError(
-                        "Cannot resolve bases for %r\nThis can happen if you are inheriting models from an "
-                        "app with migrations (e.g. contrib.auth)\n in an app with no migrations; see "
-                        "https://docs.djangoproject.com/en/%s/topics/migrations/#dependencies "
-                        "for more" % (new_unrendered_models, get_docs_version())
+                        "Cannot resolve bases for %r\nThis can happen if you are "
+                        "inheriting models from an app with migrations (e.g. "
+                        "contrib.auth)\n in an app with no migrations; see "
+                        "https://docs.djangoproject.com/en/%s/topics/migrations/"
+                        "#dependencies for more"
+                        % (new_unrendered_models, get_docs_version())
                     )
                 unrendered_models = new_unrendered_models
 
@@ -333,7 +671,6 @@ class StateApps(Apps):
         if app_label not in self.app_configs:
             self.app_configs[app_label] = AppConfigStub(app_label)
             self.app_configs[app_label].apps = self
-            self.app_configs[app_label].models = {}
         self.app_configs[app_label].models[model._meta.model_name] = model
         self.do_pending_operations(model)
         self.clear_cache()
@@ -357,37 +694,36 @@ class ModelState:
     assign new ones, as these are not detached during a clone.
     """
 
-    def __init__(self, app_label, name, fields, options=None, bases=None, managers=None):
+    def __init__(
+        self, app_label, name, fields, options=None, bases=None, managers=None
+    ):
         self.app_label = app_label
         self.name = name
-        self.fields = fields
+        self.fields = dict(fields)
         self.options = options or {}
-        self.options.setdefault('indexes', [])
-        self.options.setdefault('constraints', [])
+        self.options.setdefault("indexes", [])
+        self.options.setdefault("constraints", [])
         self.bases = bases or (models.Model,)
         self.managers = managers or []
-        # Sanity-check that fields is NOT a dict. It must be ordered.
-        if isinstance(self.fields, dict):
-            raise ValueError("ModelState.fields cannot be a dict - it must be a list of 2-tuples.")
-        for name, field in fields:
+        for name, field in self.fields.items():
             # Sanity-check that fields are NOT already bound to a model.
-            if hasattr(field, 'model'):
+            if hasattr(field, "model"):
                 raise ValueError(
                     'ModelState.fields cannot be bound to a model - "%s" is.' % name
                 )
             # Sanity-check that relation fields are NOT referring to a model class.
-            if field.is_relation and hasattr(field.related_model, '_meta'):
+            if field.is_relation and hasattr(field.related_model, "_meta"):
                 raise ValueError(
                     'ModelState.fields cannot refer to a model class - "%s.to" does. '
-                    'Use a string reference instead.' % name
+                    "Use a string reference instead." % name
                 )
-            if field.many_to_many and hasattr(field.remote_field.through, '_meta'):
+            if field.many_to_many and hasattr(field.remote_field.through, "_meta"):
                 raise ValueError(
-                    'ModelState.fields cannot refer to a model class - "%s.through" does. '
-                    'Use a string reference instead.' % name
+                    'ModelState.fields cannot refer to a model class - "%s.through" '
+                    "does. Use a string reference instead." % name
                 )
         # Sanity-check that indexes have their name set.
-        for index in self.options['indexes']:
+        for index in self.options["indexes"]:
             if not index.name:
                 raise ValueError(
                     "Indexes passed to ModelState require a name attribute. "
@@ -398,6 +734,11 @@ class ModelState:
     def name_lower(self):
         return self.name.lower()
 
+    def get_field(self, field_name):
+        if field_name == "_order":
+            field_name = self.options.get("order_with_respect_to", field_name)
+        return self.fields[field_name]
+
     @classmethod
     def from_model(cls, model, exclude_rels=False):
         """Given a model, return a ModelState representing it."""
@@ -406,28 +747,34 @@ class ModelState:
         for field in model._meta.local_fields:
             if getattr(field, "remote_field", None) and exclude_rels:
                 continue
-            if isinstance(field, OrderWrt):
+            if isinstance(field, models.OrderWrt):
                 continue
             name = field.name
             try:
                 fields.append((name, field.clone()))
             except TypeError as e:
-                raise TypeError("Couldn't reconstruct field %s on %s: %s" % (
-                    name,
-                    model._meta.label,
-                    e,
-                ))
+                raise TypeError(
+                    "Couldn't reconstruct field %s on %s: %s"
+                    % (
+                        name,
+                        model._meta.label,
+                        e,
+                    )
+                )
         if not exclude_rels:
             for field in model._meta.local_many_to_many:
                 name = field.name
                 try:
                     fields.append((name, field.clone()))
                 except TypeError as e:
-                    raise TypeError("Couldn't reconstruct m2m field %s on %s: %s" % (
-                        name,
-                        model._meta.object_name,
-                        e,
-                    ))
+                    raise TypeError(
+                        "Couldn't reconstruct m2m field %s on %s: %s"
+                        % (
+                            name,
+                            model._meta.object_name,
+                            e,
+                        )
+                    )
         # Extract the options
         options = {}
         for name in DEFAULT_NAMES:
@@ -446,9 +793,11 @@ class ModelState:
                     for index in indexes:
                         if not index.name:
                             index.set_name_with_model(model)
-                    options['indexes'] = indexes
-                elif name == 'constraints':
-                    options['constraints'] = [con.clone() for con in model._meta.constraints]
+                    options["indexes"] = indexes
+                elif name == "constraints":
+                    options["constraints"] = [
+                        con.clone() for con in model._meta.constraints
+                    ]
                 else:
                     options[name] = model._meta.original_attrs[name]
         # If we're ignoring relationships, remove all field-listing model
@@ -458,8 +807,10 @@ class ModelState:
                 if key in options:
                     del options[key]
         # Private fields are ignored, so remove options that refer to them.
-        elif options.get('order_with_respect_to') in {field.name for field in model._meta.private_fields}:
-            del options['order_with_respect_to']
+        elif options.get("order_with_respect_to") in {
+            field.name for field in model._meta.private_fields
+        }:
+            del options["order_with_respect_to"]
 
         def flatten_bases(model):
             bases = []
@@ -475,19 +826,19 @@ class ModelState:
         # __bases__ we may end up with duplicates and ordering issues, we
         # therefore discard any duplicates and reorder the bases according
         # to their index in the MRO.
-        flattened_bases = sorted(set(flatten_bases(model)), key=lambda x: model.__mro__.index(x))
+        flattened_bases = sorted(
+            set(flatten_bases(model)), key=lambda x: model.__mro__.index(x)
+        )
 
         # Make our record
         bases = tuple(
-            (
-                base._meta.label_lower
-                if hasattr(base, "_meta") else
-                base
-            )
+            (base._meta.label_lower if hasattr(base, "_meta") else base)
             for base in flattened_bases
         )
         # Ensure at least one base inherits from models.Model
-        if not any((isinstance(base, str) or issubclass(base, models.Model)) for base in bases):
+        if not any(
+            (isinstance(base, str) or issubclass(base, models.Model)) for base in bases
+        ):
             bases = (models.Model,)
 
         managers = []
@@ -514,7 +865,7 @@ class ModelState:
             managers.append((manager.name, new_manager))
 
         # Ignore a shimmed default manager called objects if it's the only one.
-        if managers == [('objects', default_manager_shim)]:
+        if managers == [("objects", default_manager_shim)]:
             managers = []
 
         # Construct the new ModelState
@@ -545,7 +896,7 @@ class ModelState:
         return self.__class__(
             app_label=self.app_label,
             name=self.name,
-            fields=list(self.fields),
+            fields=dict(self.fields),
             # Since options are shallow-copied here, operations such as
             # AddIndex must replace their option (e.g 'indexes') rather
             # than mutating it.
@@ -557,7 +908,7 @@ class ModelState:
     def render(self, apps):
         """Create a Model object from our current state into the given apps."""
         # First, make a Meta object
-        meta_contents = {'app_label': self.app_label, 'apps': apps, **self.options}
+        meta_contents = {"app_label": self.app_label, "apps": apps, **self.options}
         meta = type("Meta", (), meta_contents)
         # Then, work out our bases
         try:
@@ -566,46 +917,47 @@ class ModelState:
                 for base in self.bases
             )
         except LookupError:
-            raise InvalidBasesError("Cannot resolve one or more bases from %r" % (self.bases,))
-        # Turn fields into a dict for the body, add other bits
-        body = {name: field.clone() for name, field in self.fields}
-        body['Meta'] = meta
-        body['__module__'] = "__fake__"
+            raise InvalidBasesError(
+                "Cannot resolve one or more bases from %r" % (self.bases,)
+            )
+        # Clone fields for the body, add other bits.
+        body = {name: field.clone() for name, field in self.fields.items()}
+        body["Meta"] = meta
+        body["__module__"] = "__fake__"
 
         # Restore managers
         body.update(self.construct_managers())
         # Then, make a Model object (apps.register_model is called in __new__)
         return type(self.name, bases, body)
 
-    def get_field_by_name(self, name):
-        for fname, field in self.fields:
-            if fname == name:
-                return field
-        raise ValueError("No field called %s on model %s" % (name, self.name))
-
     def get_index_by_name(self, name):
-        for index in self.options['indexes']:
+        for index in self.options["indexes"]:
             if index.name == name:
                 return index
         raise ValueError("No index named %s on model %s" % (name, self.name))
 
     def get_constraint_by_name(self, name):
-        for constraint in self.options['constraints']:
+        for constraint in self.options["constraints"]:
             if constraint.name == name:
                 return constraint
-        raise ValueError('No constraint named %s on model %s' % (name, self.name))
+        raise ValueError("No constraint named %s on model %s" % (name, self.name))
 
     def __repr__(self):
         return "<%s: '%s.%s'>" % (self.__class__.__name__, self.app_label, self.name)
 
     def __eq__(self, other):
         return (
-            (self.app_label == other.app_label) and
-            (self.name == other.name) and
-            (len(self.fields) == len(other.fields)) and
-            all((k1 == k2 and (f1.deconstruct()[1:] == f2.deconstruct()[1:]))
-                for (k1, f1), (k2, f2) in zip(self.fields, other.fields)) and
-            (self.options == other.options) and
-            (self.bases == other.bases) and
-            (self.managers == other.managers)
+            (self.app_label == other.app_label)
+            and (self.name == other.name)
+            and (len(self.fields) == len(other.fields))
+            and all(
+                k1 == k2 and f1.deconstruct()[1:] == f2.deconstruct()[1:]
+                for (k1, f1), (k2, f2) in zip(
+                    sorted(self.fields.items()),
+                    sorted(other.fields.items()),
+                )
+            )
+            and (self.options == other.options)
+            and (self.bases == other.bases)
+            and (self.managers == other.managers)
         )
