@@ -1,22 +1,22 @@
 from django.core.exceptions import ValidationError
 from django.forms import Form
 from django.forms.fields import BooleanField, IntegerField
-from django.forms.utils import ErrorList
-from django.forms.widgets import HiddenInput, NumberInput
+from django.forms.renderers import get_default_renderer
+from django.forms.utils import ErrorList, RenderableFormMixin
+from django.forms.widgets import CheckboxInput, HiddenInput, NumberInput
 from django.utils.functional import cached_property
-from django.utils.html import html_safe
-from django.utils.safestring import mark_safe
-from django.utils.translation import gettext as _, ngettext
+from django.utils.translation import gettext_lazy as _
+from django.utils.translation import ngettext
 
-__all__ = ('BaseFormSet', 'formset_factory', 'all_valid')
+__all__ = ("BaseFormSet", "formset_factory", "all_valid")
 
 # special field names
-TOTAL_FORM_COUNT = 'TOTAL_FORMS'
-INITIAL_FORM_COUNT = 'INITIAL_FORMS'
-MIN_NUM_FORM_COUNT = 'MIN_NUM_FORMS'
-MAX_NUM_FORM_COUNT = 'MAX_NUM_FORMS'
-ORDERING_FIELD_NAME = 'ORDER'
-DELETION_FIELD_NAME = 'DELETE'
+TOTAL_FORM_COUNT = "TOTAL_FORMS"
+INITIAL_FORM_COUNT = "INITIAL_FORMS"
+MIN_NUM_FORM_COUNT = "MIN_NUM_FORMS"
+MAX_NUM_FORM_COUNT = "MAX_NUM_FORMS"
+ORDERING_FIELD_NAME = "ORDER"
+DELETION_FIELD_NAME = "DELETE"
 
 # default minimum number of forms in a formset
 DEFAULT_MIN_NUM = 0
@@ -31,26 +31,59 @@ class ManagementForm(Form):
     new forms via JavaScript, you should increment the count field of this form
     as well.
     """
+
     def __init__(self, *args, **kwargs):
         self.base_fields[TOTAL_FORM_COUNT] = IntegerField(widget=HiddenInput)
         self.base_fields[INITIAL_FORM_COUNT] = IntegerField(widget=HiddenInput)
         # MIN_NUM_FORM_COUNT and MAX_NUM_FORM_COUNT are output with the rest of
         # the management form, but only for the convenience of client-side
         # code. The POST value of them returned from the client is not checked.
-        self.base_fields[MIN_NUM_FORM_COUNT] = IntegerField(required=False, widget=HiddenInput)
-        self.base_fields[MAX_NUM_FORM_COUNT] = IntegerField(required=False, widget=HiddenInput)
+        self.base_fields[MIN_NUM_FORM_COUNT] = IntegerField(
+            required=False, widget=HiddenInput
+        )
+        self.base_fields[MAX_NUM_FORM_COUNT] = IntegerField(
+            required=False, widget=HiddenInput
+        )
         super().__init__(*args, **kwargs)
 
+    def clean(self):
+        cleaned_data = super().clean()
+        # When the management form is invalid, we don't know how many forms
+        # were submitted.
+        cleaned_data.setdefault(TOTAL_FORM_COUNT, 0)
+        cleaned_data.setdefault(INITIAL_FORM_COUNT, 0)
+        return cleaned_data
 
-@html_safe
-class BaseFormSet:
+
+class BaseFormSet(RenderableFormMixin):
     """
     A collection of instances of the same Form class.
     """
-    ordering_widget = NumberInput
 
-    def __init__(self, data=None, files=None, auto_id='id_%s', prefix=None,
-                 initial=None, error_class=ErrorList, form_kwargs=None):
+    deletion_widget = CheckboxInput
+    ordering_widget = NumberInput
+    default_error_messages = {
+        "missing_management_form": _(
+            "ManagementForm data is missing or has been tampered with. Missing fields: "
+            "%(field_names)s. You may need to file a bug report if the issue persists."
+        ),
+    }
+    template_name = "django/forms/formsets/default.html"
+    template_name_p = "django/forms/formsets/p.html"
+    template_name_table = "django/forms/formsets/table.html"
+    template_name_ul = "django/forms/formsets/ul.html"
+
+    def __init__(
+        self,
+        data=None,
+        files=None,
+        auto_id="id_%s",
+        prefix=None,
+        initial=None,
+        error_class=ErrorList,
+        form_kwargs=None,
+        error_messages=None,
+    ):
         self.is_bound = data is not None or files is not None
         self.prefix = prefix or self.get_default_prefix()
         self.auto_id = auto_id
@@ -62,8 +95,12 @@ class BaseFormSet:
         self._errors = None
         self._non_form_errors = None
 
-    def __str__(self):
-        return self.as_table()
+        messages = {}
+        for cls in reversed(type(self).__mro__):
+            messages.update(getattr(cls, "default_error_messages", {}))
+        if error_messages is not None:
+            messages.update(error_messages)
+        self.error_messages = messages
 
     def __iter__(self):
         """Yield the forms in the order they should be rendered."""
@@ -87,19 +124,25 @@ class BaseFormSet:
     def management_form(self):
         """Return the ManagementForm instance for this FormSet."""
         if self.is_bound:
-            form = ManagementForm(self.data, auto_id=self.auto_id, prefix=self.prefix)
-            if not form.is_valid():
-                raise ValidationError(
-                    _('ManagementForm data is missing or has been tampered with'),
-                    code='missing_management_form',
-                )
+            form = ManagementForm(
+                self.data,
+                auto_id=self.auto_id,
+                prefix=self.prefix,
+                renderer=self.renderer,
+            )
+            form.full_clean()
         else:
-            form = ManagementForm(auto_id=self.auto_id, prefix=self.prefix, initial={
-                TOTAL_FORM_COUNT: self.total_form_count(),
-                INITIAL_FORM_COUNT: self.initial_form_count(),
-                MIN_NUM_FORM_COUNT: self.min_num,
-                MAX_NUM_FORM_COUNT: self.max_num
-            })
+            form = ManagementForm(
+                auto_id=self.auto_id,
+                prefix=self.prefix,
+                initial={
+                    TOTAL_FORM_COUNT: self.total_form_count(),
+                    INITIAL_FORM_COUNT: self.initial_form_count(),
+                    MIN_NUM_FORM_COUNT: self.min_num,
+                    MAX_NUM_FORM_COUNT: self.max_num,
+                },
+                renderer=self.renderer,
+            )
         return form
 
     def total_form_count(self):
@@ -109,7 +152,9 @@ class BaseFormSet:
             # count in the data; this is DoS protection to prevent clients
             # from forcing the server to instantiate arbitrary numbers of
             # forms
-            return min(self.management_form.cleaned_data[TOTAL_FORM_COUNT], self.absolute_max)
+            return min(
+                self.management_form.cleaned_data[TOTAL_FORM_COUNT], self.absolute_max
+            )
         else:
             initial_forms = self.initial_form_count()
             total_forms = max(initial_forms, self.min_num) + self.extra
@@ -151,26 +196,27 @@ class BaseFormSet:
     def _construct_form(self, i, **kwargs):
         """Instantiate and return the i-th form instance in a formset."""
         defaults = {
-            'auto_id': self.auto_id,
-            'prefix': self.add_prefix(i),
-            'error_class': self.error_class,
+            "auto_id": self.auto_id,
+            "prefix": self.add_prefix(i),
+            "error_class": self.error_class,
             # Don't render the HTML 'required' attribute as it may cause
             # incorrect validation for extra, optional, and deleted
             # forms in the formset.
-            'use_required_attribute': False,
+            "use_required_attribute": False,
+            "renderer": self.renderer,
         }
         if self.is_bound:
-            defaults['data'] = self.data
-            defaults['files'] = self.files
-        if self.initial and 'initial' not in kwargs:
+            defaults["data"] = self.data
+            defaults["files"] = self.files
+        if self.initial and "initial" not in kwargs:
             try:
-                defaults['initial'] = self.initial[i]
+                defaults["initial"] = self.initial[i]
             except IndexError:
                 pass
         # Allow extra forms to be empty, unless they're part of
         # the minimum forms.
         if i >= self.initial_form_count() and i >= self.min_num:
-            defaults['empty_permitted'] = True
+            defaults["empty_permitted"] = True
         defaults.update(kwargs)
         form = self.form(**defaults)
         self.add_fields(form, i)
@@ -179,21 +225,22 @@ class BaseFormSet:
     @property
     def initial_forms(self):
         """Return a list of all the initial forms in this formset."""
-        return self.forms[:self.initial_form_count()]
+        return self.forms[: self.initial_form_count()]
 
     @property
     def extra_forms(self):
         """Return a list of all the extra forms in this formset."""
-        return self.forms[self.initial_form_count():]
+        return self.forms[self.initial_form_count() :]
 
     @property
     def empty_form(self):
         form = self.form(
             auto_id=self.auto_id,
-            prefix=self.add_prefix('__prefix__'),
+            prefix=self.add_prefix("__prefix__"),
             empty_permitted=True,
             use_required_attribute=False,
-            **self.get_form_kwargs(None)
+            **self.get_form_kwargs(None),
+            renderer=self.renderer,
         )
         self.add_fields(form, None)
         return form
@@ -204,7 +251,9 @@ class BaseFormSet:
         Return a list of form.cleaned_data dicts for every form in self.forms.
         """
         if not self.is_valid():
-            raise AttributeError("'%s' object has no attribute 'cleaned_data'" % self.__class__.__name__)
+            raise AttributeError(
+                "'%s' object has no attribute 'cleaned_data'" % self.__class__.__name__
+            )
         return [form.cleaned_data for form in self.forms]
 
     @property
@@ -214,10 +263,9 @@ class BaseFormSet:
             return []
         # construct _deleted_form_indexes which is just a list of form indexes
         # that have had their deletion widget set to True
-        if not hasattr(self, '_deleted_form_indexes'):
+        if not hasattr(self, "_deleted_form_indexes"):
             self._deleted_form_indexes = []
-            for i in range(0, self.total_form_count()):
-                form = self.forms[i]
+            for i, form in enumerate(self.forms):
                 # if this is an extra form and hasn't changed, don't consider it
                 if i >= self.initial_form_count() and not form.has_changed():
                     continue
@@ -232,15 +280,16 @@ class BaseFormSet:
         Raise an AttributeError if ordering is not allowed.
         """
         if not self.is_valid() or not self.can_order:
-            raise AttributeError("'%s' object has no attribute 'ordered_forms'" % self.__class__.__name__)
+            raise AttributeError(
+                "'%s' object has no attribute 'ordered_forms'" % self.__class__.__name__
+            )
         # Construct _ordering, which is a list of (form_index, order_field_value)
         # tuples. After constructing this list, we'll sort it by order_field_value
         # so we have a way to get to the form indexes in the order specified
         # by the form data.
-        if not hasattr(self, '_ordering'):
+        if not hasattr(self, "_ordering"):
             self._ordering = []
-            for i in range(0, self.total_form_count()):
-                form = self.forms[i]
+            for i, form in enumerate(self.forms):
                 # if this is an extra form and hasn't changed, don't consider it
                 if i >= self.initial_form_count() and not form.has_changed():
                     continue
@@ -258,6 +307,7 @@ class BaseFormSet:
                 if k[1] is None:
                     return (1, 0)  # +infinity, larger than any number
                 return (0, k[1])
+
             self._ordering.sort(key=compare_ordering_key)
         # Return a list of form.cleaned_data dicts in the order specified by
         # the form data.
@@ -265,7 +315,11 @@ class BaseFormSet:
 
     @classmethod
     def get_default_prefix(cls):
-        return 'form'
+        return "form"
+
+    @classmethod
+    def get_deletion_widget(cls):
+        return cls.deletion_widget
 
     @classmethod
     def get_ordering_widget(cls):
@@ -290,8 +344,9 @@ class BaseFormSet:
 
     def total_error_count(self):
         """Return the number of errors across all forms in the formset."""
-        return len(self.non_form_errors()) +\
-            sum(len(form_errors) for form_errors in self.errors)
+        return len(self.non_form_errors()) + sum(
+            len(form_errors) for form_errors in self.errors
+        )
 
     def _should_delete_form(self, form):
         """Return whether or not the form was marked for deletion."""
@@ -301,18 +356,17 @@ class BaseFormSet:
         """Return True if every form in self.forms is valid."""
         if not self.is_bound:
             return False
-        # We loop over every form.errors here rather than short circuiting on the
-        # first failure to make sure validation gets triggered for every form.
-        forms_valid = True
-        # This triggers a full clean.
+        # Accessing errors triggers a full clean the first time only.
         self.errors
-        for i in range(0, self.total_form_count()):
-            form = self.forms[i]
-            if self.can_delete and self._should_delete_form(form):
-                # This form is going to be deleted so any of its errors
-                # shouldn't cause the entire formset to be invalid.
-                continue
-            forms_valid &= form.is_valid()
+        # List comprehension ensures is_valid() is called for all forms.
+        # Forms due to be deleted shouldn't cause the formset to be invalid.
+        forms_valid = all(
+            [
+                form.is_valid()
+                for form in self.forms
+                if not (self.can_delete and self._should_delete_form(form))
+            ]
+        )
         return forms_valid and not self.non_form_errors()
 
     def full_clean(self):
@@ -321,13 +375,28 @@ class BaseFormSet:
         self._non_form_errors.
         """
         self._errors = []
-        self._non_form_errors = self.error_class()
+        self._non_form_errors = self.error_class(
+            error_class="nonform", renderer=self.renderer
+        )
         empty_forms_count = 0
 
         if not self.is_bound:  # Stop further processing.
             return
-        for i in range(0, self.total_form_count()):
-            form = self.forms[i]
+
+        if not self.management_form.is_valid():
+            error = ValidationError(
+                self.error_messages["missing_management_form"],
+                params={
+                    "field_names": ", ".join(
+                        self.management_form.add_prefix(field_name)
+                        for field_name in self.management_form.errors
+                    ),
+                },
+                code="missing_management_form",
+            )
+            self._non_form_errors.append(error)
+
+        for i, form in enumerate(self.forms):
             # Empty forms are unchanged forms beyond those with initial data.
             if not form.has_changed() and i >= self.initial_form_count():
                 empty_forms_count += 1
@@ -338,24 +407,45 @@ class BaseFormSet:
                 continue
             self._errors.append(form_errors)
         try:
-            if (self.validate_max and
-                    self.total_form_count() - len(self.deleted_forms) > self.max_num) or \
-                    self.management_form.cleaned_data[TOTAL_FORM_COUNT] > self.absolute_max:
-                raise ValidationError(ngettext(
-                    "Please submit %d or fewer forms.",
-                    "Please submit %d or fewer forms.", self.max_num) % self.max_num,
-                    code='too_many_forms',
+            if (
+                self.validate_max
+                and self.total_form_count() - len(self.deleted_forms) > self.max_num
+            ) or self.management_form.cleaned_data[
+                TOTAL_FORM_COUNT
+            ] > self.absolute_max:
+                raise ValidationError(
+                    ngettext(
+                        "Please submit at most %d form.",
+                        "Please submit at most %d forms.",
+                        self.max_num,
+                    )
+                    % self.max_num,
+                    code="too_many_forms",
+                )
+            if (
+                self.validate_min
+                and self.total_form_count()
+                - len(self.deleted_forms)
+                - empty_forms_count
+                < self.min_num
+            ):
+                raise ValidationError(
+                    ngettext(
+                        "Please submit at least %d form.",
+                        "Please submit at least %d forms.",
+                        self.min_num,
+                    )
+                    % self.min_num,
+                    code="too_few_forms",
                 )
-            if (self.validate_min and
-                    self.total_form_count() - len(self.deleted_forms) - empty_forms_count < self.min_num):
-                raise ValidationError(ngettext(
-                    "Please submit %d or more forms.",
-                    "Please submit %d or more forms.", self.min_num) % self.min_num,
-                    code='too_few_forms')
             # Give self.clean() a chance to do cross-form validation.
             self.clean()
         except ValidationError as e:
-            self._non_form_errors = self.error_class(e.error_list)
+            self._non_form_errors = self.error_class(
+                e.error_list,
+                error_class="nonform",
+                renderer=self.renderer,
+            )
 
     def clean(self):
         """
@@ -372,26 +462,31 @@ class BaseFormSet:
 
     def add_fields(self, form, index):
         """A hook for adding extra fields on to each form instance."""
+        initial_form_count = self.initial_form_count()
         if self.can_order:
             # Only pre-fill the ordering field for initial forms.
-            if index is not None and index < self.initial_form_count():
+            if index is not None and index < initial_form_count:
                 form.fields[ORDERING_FIELD_NAME] = IntegerField(
-                    label=_('Order'),
+                    label=_("Order"),
                     initial=index + 1,
                     required=False,
                     widget=self.get_ordering_widget(),
                 )
             else:
                 form.fields[ORDERING_FIELD_NAME] = IntegerField(
-                    label=_('Order'),
+                    label=_("Order"),
                     required=False,
                     widget=self.get_ordering_widget(),
                 )
-        if self.can_delete:
-            form.fields[DELETION_FIELD_NAME] = BooleanField(label=_('Delete'), required=False)
+        if self.can_delete and (self.can_delete_extra or index < initial_form_count):
+            form.fields[DELETION_FIELD_NAME] = BooleanField(
+                label=_("Delete"),
+                required=False,
+                widget=self.get_deletion_widget(),
+            )
 
     def add_prefix(self, index):
-        return '%s-%s' % (self.prefix, index)
+        return "%s-%s" % (self.prefix, index)
 
     def is_multipart(self):
         """
@@ -412,54 +507,53 @@ class BaseFormSet:
         else:
             return self.empty_form.media
 
-    def as_table(self):
-        "Return this formset rendered as HTML <tr>s -- excluding the <table></table>."
-        # XXX: there is no semantic division between forms here, there
-        # probably should be. It might make sense to render each form as a
-        # table row with each field as a td.
-        forms = ' '.join(form.as_table() for form in self)
-        return mark_safe(str(self.management_form) + '\n' + forms)
-
-    def as_p(self):
-        "Return this formset rendered as HTML <p>s."
-        forms = ' '.join(form.as_p() for form in self)
-        return mark_safe(str(self.management_form) + '\n' + forms)
-
-    def as_ul(self):
-        "Return this formset rendered as HTML <li>s."
-        forms = ' '.join(form.as_ul() for form in self)
-        return mark_safe(str(self.management_form) + '\n' + forms)
+    def get_context(self):
+        return {"formset": self}
 
 
-def formset_factory(form, formset=BaseFormSet, extra=1, can_order=False,
-                    can_delete=False, max_num=None, validate_max=False,
-                    min_num=None, validate_min=False):
+def formset_factory(
+    form,
+    formset=BaseFormSet,
+    extra=1,
+    can_order=False,
+    can_delete=False,
+    max_num=None,
+    validate_max=False,
+    min_num=None,
+    validate_min=False,
+    absolute_max=None,
+    can_delete_extra=True,
+    renderer=None,
+):
     """Return a FormSet for the given form class."""
     if min_num is None:
         min_num = DEFAULT_MIN_NUM
     if max_num is None:
         max_num = DEFAULT_MAX_NUM
-    # hard limit on forms instantiated, to prevent memory-exhaustion attacks
-    # limit is simply max_num + DEFAULT_MAX_NUM (which is 2*DEFAULT_MAX_NUM
-    # if max_num is None in the first place)
-    absolute_max = max_num + DEFAULT_MAX_NUM
+    # absolute_max is a hard limit on forms instantiated, to prevent
+    # memory-exhaustion attacks. Default to max_num + DEFAULT_MAX_NUM
+    # (which is 2 * DEFAULT_MAX_NUM if max_num is None in the first place).
+    if absolute_max is None:
+        absolute_max = max_num + DEFAULT_MAX_NUM
+    if max_num > absolute_max:
+        raise ValueError("'absolute_max' must be greater or equal to 'max_num'.")
     attrs = {
-        'form': form,
-        'extra': extra,
-        'can_order': can_order,
-        'can_delete': can_delete,
-        'min_num': min_num,
-        'max_num': max_num,
-        'absolute_max': absolute_max,
-        'validate_min': validate_min,
-        'validate_max': validate_max,
+        "form": form,
+        "extra": extra,
+        "can_order": can_order,
+        "can_delete": can_delete,
+        "can_delete_extra": can_delete_extra,
+        "min_num": min_num,
+        "max_num": max_num,
+        "absolute_max": absolute_max,
+        "validate_min": validate_min,
+        "validate_max": validate_max,
+        "renderer": renderer or get_default_renderer(),
     }
-    return type(form.__name__ + 'FormSet', (formset,), attrs)
+    return type(form.__name__ + "FormSet", (formset,), attrs)
 
 
 def all_valid(formsets):
     """Validate every formset and return True if all are valid."""
-    valid = True
-    for formset in formsets:
-        valid &= formset.is_valid()
-    return valid
+    # List comprehension ensures is_valid() is called for all formsets.
+    return all([formset.is_valid() for formset in formsets])
