@@ -7,14 +7,6 @@ from django.template.loader import get_template
 from django.utils.functional import cached_property
 from django.utils.module_loading import import_string
 
-try:
-    from django.template.backends.jinja2 import Jinja2
-except ImportError:
-    def Jinja2(params):
-        raise ImportError("jinja2 isn't installed")
-
-ROOT = Path(__file__).parent
-
 
 @functools.lru_cache()
 def get_default_renderer():
@@ -24,7 +16,7 @@ def get_default_renderer():
 
 class BaseRenderer:
     def get_template(self, template_name):
-        raise NotImplementedError('subclasses must implement get_template()')
+        raise NotImplementedError("subclasses must implement get_template()")
 
     def render(self, template_name, context, request=None):
         template = self.get_template(template_name)
@@ -37,12 +29,14 @@ class EngineMixin:
 
     @cached_property
     def engine(self):
-        return self.backend({
-            'APP_DIRS': True,
-            'DIRS': [str(ROOT / self.backend.app_dirname)],
-            'NAME': 'djangoforms',
-            'OPTIONS': {},
-        })
+        return self.backend(
+            {
+                "APP_DIRS": True,
+                "DIRS": [Path(__file__).parent / self.backend.app_dirname],
+                "NAME": "djangoforms",
+                "OPTIONS": {},
+            }
+        )
 
 
 class DjangoTemplates(EngineMixin, BaseRenderer):
@@ -50,6 +44,7 @@ class DjangoTemplates(EngineMixin, BaseRenderer):
     Load Django templates from the built-in widget templates in
     django/forms/templates and from apps' 'templates' directory.
     """
+
     backend = DjangoTemplates
 
 
@@ -58,7 +53,12 @@ class Jinja2(EngineMixin, BaseRenderer):
     Load Jinja2 templates from the built-in widget templates in
     django/forms/jinja2 and from apps' 'jinja2' directory.
     """
-    backend = Jinja2
+
+    @cached_property
+    def backend(self):
+        from django.template.backends.jinja2 import Jinja2
+
+        return Jinja2
 
 
 class TemplatesSetting(BaseRenderer):
@@ -66,5 +66,6 @@ class TemplatesSetting(BaseRenderer):
     Load templates using template.loader.get_template() which is configured
     based on settings.TEMPLATES.
     """
+
     def get_template(self, template_name):
         return get_template(template_name)
