@@ -1,26 +1,37 @@
 import copy
 import os
+import sys
 from importlib import import_module
 from importlib.util import find_spec as importlib_find
 
 
+def cached_import(module_path, class_name):
+    modules = sys.modules
+    if module_path not in modules or (
+        # Module is not fully initialized.
+        getattr(modules[module_path], "__spec__", None) is not None
+        and getattr(modules[module_path].__spec__, "_initializing", False) is True
+    ):
+        import_module(module_path)
+    return getattr(modules[module_path], class_name)
+
+
 def import_string(dotted_path):
     """
     Import a dotted module path and return the attribute/class designated by the
     last name in the path. Raise ImportError if the import failed.
     """
     try:
-        module_path, class_name = dotted_path.rsplit('.', 1)
+        module_path, class_name = dotted_path.rsplit(".", 1)
     except ValueError as err:
         raise ImportError("%s doesn't look like a module path" % dotted_path) from err
 
-    module = import_module(module_path)
-
     try:
-        return getattr(module, class_name)
+        return cached_import(module_path, class_name)
     except AttributeError as err:
-        raise ImportError('Module "%s" does not define a "%s" attribute/class' % (
-            module_path, class_name)
+        raise ImportError(
+            'Module "%s" does not define a "%s" attribute/class'
+            % (module_path, class_name)
         ) from err
 
 
@@ -36,7 +47,7 @@ def autodiscover_modules(*args, **kwargs):
     """
     from django.apps import apps
 
-    register_to = kwargs.get('register_to')
+    register_to = kwargs.get("register_to")
     for app_config in apps.get_app_configs():
         for module_to_search in args:
             # Attempt to import the app's module.
@@ -44,7 +55,7 @@ def autodiscover_modules(*args, **kwargs):
                 if register_to:
                     before_import_registry = copy.copy(register_to._registry)
 
-                import_module('%s.%s' % (app_config.name, module_to_search))
+                import_module("%s.%s" % (app_config.name, module_to_search))
             except Exception:
                 # Reset the registry to the state before the last import
                 # as this import will have to reoccur on the next request and
@@ -69,14 +80,12 @@ def module_has_submodule(package, module_name):
         # package isn't a package.
         return False
 
-    full_module_name = package_name + '.' + module_name
+    full_module_name = package_name + "." + module_name
     try:
         return importlib_find(full_module_name, package_path) is not None
-    except (ImportError, AttributeError):
-        # When module_name is an invalid dotted path, Python raises ImportError
-        # (or ModuleNotFoundError in Python 3.6+). AttributeError may be raised
-        # if the penultimate part of the path is not a package.
-        # (http://bugs.python.org/issue30436)
+    except ModuleNotFoundError:
+        # When module_name is an invalid dotted path, Python raises
+        # ModuleNotFoundError.
         return False
 
 
@@ -87,12 +96,12 @@ def module_dir(module):
     Raise ValueError otherwise, e.g. for namespace packages that are split
     over several directories.
     """
-    # Convert to list because _NamespacePath does not support indexing on 3.3.
-    paths = list(getattr(module, '__path__', []))
+    # Convert to list because __path__ may not support indexing.
+    paths = list(getattr(module, "__path__", []))
     if len(paths) == 1:
         return paths[0]
     else:
-        filename = getattr(module, '__file__', None)
+        filename = getattr(module, "__file__", None)
         if filename is not None:
             return os.path.dirname(filename)
     raise ValueError("Cannot determine directory containing %s" % module)
